# Copyright 2022 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""The implementation of `tf.data.Dataset.from_tensor_slices`."""

from tensorflow.python.data.ops import dataset_ops
from tensorflow.python.data.util import nest
from tensorflow.python.data.util import structure
from tensorflow.python.framework import tensor_shape
from tensorflow.python.ops import gen_dataset_ops


def _from_tensor_slices(tensors, name=None):
  return _TensorSliceDataset(tensors, name=name)


class _TensorSliceDataset(dataset_ops.DatasetSource):
  """A `Dataset` of slices from a dataset element."""

  def __init__(self, element, is_files=False, name=None):
    """See `Dataset.from_tensor_slices` for details."""
    element = structure.normalize_element(element)
    batched_spec = structure.type_spec_from_value(element)
    self._tensors = structure.to_batched_tensor_list(batched_spec, element)
    if not self._tensors:
      raise ValueError("Invalid `element`. `element` should not be empty.")
    self._structure = nest.map_structure(
        lambda component_spec: component_spec._unbatch(), batched_spec)  # pylint: disable=protected-access
    self._name = name

    batch_dim = tensor_shape.Dimension(
        tensor_shape.dimension_value(self._tensors[0].get_shape()[0]))
    for t in self._tensors[1:]:
      batch_dim.assert_is_compatible_with(
          tensor_shape.Dimension(
              tensor_shape.dimension_value(t.get_shape()[0])))

    variant_tensor = gen_dataset_ops.tensor_slice_dataset(
        self._tensors,
        output_shapes=structure.get_flat_tensor_shapes(self._structure),
        is_files=is_files,
        metadata=self._metadata.SerializeToString())
    super().__init__(variant_tensor)

  @property
  def element_spec(self):
    return self._structure



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/compiler/tf2tensorrt/utils/trt_engine_instance.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import tensor_shape_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?tensorflow/compiler/tf2tensorrt/utils/trt_engine_instance.proto\x12\x13tensorflow.tensorrt\x1a,tensorflow/core/framework/tensor_shape.proto\"b\n\x11TRTEngineInstance\x12\x32\n\x0cinput_shapes\x18\x01 \x03(\x0b\x32\x1c.tensorflow.TensorShapeProto\x12\x19\n\x11serialized_engine\x18\x02 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.compiler.tf2tensorrt.utils.trt_engine_instance_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _TRTENGINEINSTANCE._serialized_start=134
  _TRTENGINEINSTANCE._serialized_end=232
# @@protoc_insertion_point(module_scope)



# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Experimental API for TensorFlow's "Eager" mode of execution."""

from tensorflow.python import pywrap_tfe
from tensorflow.python.framework import errors

# Trace of execution and memory usage.
_active_trace = None


def _status_to_exception(status):
  try:
    error_class = errors.exception_type_from_error_code(status.code)
    return error_class(None, None, status.message, status.payloads)
  except KeyError:
    return errors.UnknownError(None, None, status.message, status.code,
                               status.payloads)


class _NotOkStatusException(Exception):
  """Exception class to handle not ok Status."""

  def __init__(self, message, code, payloads):
    super(_NotOkStatusException, self).__init__()
    self.message = message
    self.code = code
    self.payloads = payloads

  def __str__(self):
    e = _status_to_exception(self)
    return "%s: %s" % (e.__class__.__name__, e)


pywrap_tfe.TFE_Py_RegisterExceptionClass(_NotOkStatusException)


class _FallbackException(Exception):
  """Exception class to handle fallback from the fastpath.

  The fastpath that we refer to here is the one implemented to reduce per-op
  overheads (TFE_Py_FastPathExecute_C). If the conditions for executing the op
  on the fastpath are not met, we fallback to a safer (and more complete)
  slowpath, and this Exception is raised to signal that transition.
  """
  pass


class _SymbolicException(Exception):
  """Exception class to handle use of symbolic tensors when executing eagerly.

  `keras.Input()` creates symbolic tensors (in a FuncGraph managed by the
  Keras backend) while in eager execution. This exception is used to
  identify this case (raised in `convert_to_tensor` cause generated functions
  for ops to construct graphs instead of executing the kernel).
  """
  pass


pywrap_tfe.TFE_Py_RegisterFallbackExceptionClass(_FallbackException)



# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Ignore_errors dataset transformations."""
from tensorflow.python.util import deprecation
from tensorflow.python.util.tf_export import tf_export


@tf_export("data.experimental.ignore_errors")
@deprecation.deprecated(None, "Use `tf.data.Dataset.ignore_errors` instead.")
def ignore_errors(log_warning=False):
  """Creates a `Dataset` from another `Dataset` and silently ignores any errors.

  Use this transformation to produce a dataset that contains the same elements
  as the input, but silently drops any elements that caused an error. For
  example:

  ```python
  dataset = tf.data.Dataset.from_tensor_slices([1., 2., 0., 4.])

  # Computing `tf.debugging.check_numerics(1. / 0.)` will raise an
  InvalidArgumentError.
  dataset = dataset.map(lambda x: tf.debugging.check_numerics(1. / x, "error"))

  # Using `ignore_errors()` will drop the element that causes an error.
  dataset =
      dataset.apply(tf.data.experimental.ignore_errors())  # ==> {1., 0.5, 0.2}
  ```
  Args:
     log_warning: (Optional.) A 'tf.bool' scalar indicating whether ignored
      errors should be logged to stderr. Defaults to 'False'.

  Returns:
    A `Dataset` transformation function, which can be passed to
    `tf.data.Dataset.apply`.
  """
  def _apply_fn(dataset):
    return dataset.ignore_errors(log_warning)

  return _apply_fn



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/function/polymorphism/function_type.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.function.trace_type import serialization_pb2 as tensorflow_dot_core_dot_function_dot_trace__type_dot_serialization__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9tensorflow/core/function/polymorphism/function_type.proto\x12\x33tensorflow.core.function.polymorphism.function_type\x1a\x37tensorflow/core/function/trace_type/serialization.proto\"\xe0\x02\n\tParameter\x12\x0c\n\x04name\x18\x01 \x01(\t\x12Q\n\x04kind\x18\x02 \x01(\x0e\x32\x43.tensorflow.core.function.polymorphism.function_type.Parameter.Kind\x12\x13\n\x0bis_optional\x18\x03 \x01(\x08\x12_\n\x0ftype_constraint\x18\x04 \x01(\x0b\x32\x46.tensorflow.core.function.trace_type.serialization.SerializedTraceType\"|\n\x04Kind\x12\r\n\tUNDEFINED\x10\x00\x12\x13\n\x0fPOSITIONAL_ONLY\x10\x01\x12\x19\n\x15POSITIONAL_OR_KEYWORD\x10\x02\x12\x12\n\x0eVAR_POSITIONAL\x10\x03\x12\x10\n\x0cKEYWORD_ONLY\x10\x04\x12\x0f\n\x0bVAR_KEYWORD\x10\x05\"x\n\x07\x43\x61pture\x12\x0c\n\x04name\x18\x01 \x01(\t\x12_\n\x0ftype_constraint\x18\x02 \x01(\x0b\x32\x46.tensorflow.core.function.trace_type.serialization.SerializedTraceType\"\xb2\x01\n\x0c\x46unctionType\x12R\n\nparameters\x18\x01 \x03(\x0b\x32>.tensorflow.core.function.polymorphism.function_type.Parameter\x12N\n\x08\x63\x61ptures\x18\x02 \x03(\x0b\x32<.tensorflow.core.function.polymorphism.function_type.Capture')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.function.polymorphism.function_type_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _PARAMETER._serialized_start=172
  _PARAMETER._serialized_end=524
  _PARAMETER_KIND._serialized_start=400
  _PARAMETER_KIND._serialized_end=524
  _CAPTURE._serialized_start=526
  _CAPTURE._serialized_end=646
  _FUNCTIONTYPE._serialized_start=649
  _FUNCTIONTYPE._serialized_end=827
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/debug/debugger_event_metadata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3tensorflow/core/debug/debugger_event_metadata.proto\x12!third_party.tensorflow.core.debug\"e\n\x15\x44\x65\x62uggerEventMetadata\x12\x0e\n\x06\x64\x65vice\x18\x01 \x01(\t\x12\x13\n\x0boutput_slot\x18\x02 \x01(\x05\x12\x12\n\nnum_chunks\x18\x03 \x01(\x05\x12\x13\n\x0b\x63hunk_index\x18\x04 \x01(\x05\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.debug.debugger_event_metadata_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DEBUGGEREVENTMETADATA._serialized_start=90
  _DEBUGGEREVENTMETADATA._serialized_end=191
# @@protoc_insertion_point(module_scope)



"""Python wrappers around TensorFlow ops.

This file is MACHINE GENERATED! Do not edit.
"""

import collections

from tensorflow.python import pywrap_tfe as pywrap_tfe
from tensorflow.python.eager import context as _context
from tensorflow.python.eager import core as _core
from tensorflow.python.eager import execute as _execute
from tensorflow.python.framework import dtypes as _dtypes

from tensorflow.python.framework import op_def_registry as _op_def_registry
from tensorflow.python.framework import ops as _ops
from tensorflow.python.framework import op_def_library as _op_def_library
from tensorflow.python.util.deprecation import deprecated_endpoints
from tensorflow.python.util import dispatch as _dispatch
from tensorflow.python.util.tf_export import tf_export

from typing import TypeVar

def uniform_dequantize(input, scales, zero_points, Tout, quantization_min_val, quantization_max_val, quantization_axis=-1, name=None):
  r"""Perform dequantization on the quantized Tensor `input`.

  Given quantized `input` which was quantized using `scales` and `zero_points`, performs dequantization using the formula:
  dequantized_data = (quantized_data - zero_point) * scale.

  Args:
    input: A `Tensor`. Must be one of the following types: `qint8`, `qint32`.
      Must be a Tensor of Tin.
    scales: A `Tensor` of type `float32`.
      The float value(s) used as scale(s) when quantizing original data that input represents.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (input.dim_size(quantization_axis),) (per-axis quantization).
    zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point(s) when quantizing original data that input represents.
      Same shape condition as scales.
    Tout: A `tf.DType` from: `tf.float32`.
      The type of output Tensor. A tf.DType from: tf.qint8, tf.qint32
    quantization_min_val: An `int`.
      The quantization min value that was used when input was quantized.
      The purpose of this attribute is typically (but not limited to) to indicate narrow range, where this is set to:
      `(Tin lowest) + 1` if narrow range, and `(Tin lowest)` otherwise.
      For example, if Tin is qint8, this is set to -127 if narrow range quantized or -128 if not.
    quantization_max_val: An `int`.
      The quantization max value that was used when input was quantized.
      The purpose of this attribute is typically (but not limited to) indicate narrow range, where this is set to:
      `(Tout max)` for both narrow range and not narrow range.
      For example, if Tin is qint8, this is set to 127.
    quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization. Otherwise, it must be set within range [0, input.dims()).
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformDequantize", name, input, scales, zero_points, "Tout",
        Tout, "quantization_axis", quantization_axis, "quantization_min_val",
        quantization_min_val, "quantization_max_val", quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_dequantize_eager_fallback(
          input, scales, zero_points, Tout=Tout,
          quantization_axis=quantization_axis,
          quantization_min_val=quantization_min_val,
          quantization_max_val=quantization_max_val, name=name, ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  quantization_min_val = _execute.make_int(quantization_min_val, "quantization_min_val")
  quantization_max_val = _execute.make_int(quantization_max_val, "quantization_max_val")
  if quantization_axis is None:
    quantization_axis = -1
  quantization_axis = _execute.make_int(quantization_axis, "quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformDequantize", input=input, scales=scales,
                             zero_points=zero_points, Tout=Tout,
                             quantization_min_val=quantization_min_val,
                             quantization_max_val=quantization_max_val,
                             quantization_axis=quantization_axis, name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tin", _op._get_attr_type("Tin"), "Tout",
              _op._get_attr_type("Tout"), "quantization_axis",
              _op._get_attr_int("quantization_axis"), "quantization_min_val",
              _op._get_attr_int("quantization_min_val"),
              "quantization_max_val",
              _op._get_attr_int("quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformDequantize", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformDequantize = tf_export("raw_ops.UniformDequantize")(_ops.to_raw_op(uniform_dequantize))


def uniform_dequantize_eager_fallback(input, scales, zero_points, Tout, quantization_min_val, quantization_max_val, quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  quantization_min_val = _execute.make_int(quantization_min_val, "quantization_min_val")
  quantization_max_val = _execute.make_int(quantization_max_val, "quantization_max_val")
  if quantization_axis is None:
    quantization_axis = -1
  quantization_axis = _execute.make_int(quantization_axis, "quantization_axis")
  _attr_Tin, (input,) = _execute.args_to_matching_eager([input], ctx, [_dtypes.qint8, _dtypes.qint32, ])
  scales = _ops.convert_to_tensor(scales, _dtypes.float32)
  zero_points = _ops.convert_to_tensor(zero_points, _dtypes.int32)
  _inputs_flat = [input, scales, zero_points]
  _attrs = ("Tin", _attr_Tin, "Tout", Tout, "quantization_axis",
  quantization_axis, "quantization_min_val", quantization_min_val,
  "quantization_max_val", quantization_max_val)
  _result = _execute.execute(b"UniformDequantize", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=ctx, name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformDequantize", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantize(input, scales, zero_points, Tout, quantization_min_val, quantization_max_val, quantization_axis=-1, name=None):
  r"""Perform quantization on Tensor `input`.

  Given `input`, `scales` and `zero_points`, performs quantization using the formula:
  quantized_data = floor(input_data * (1.0f / scale) + 0.5f) + zero_point

  Args:
    input: A `Tensor`. Must be one of the following types: `float32`.
      Must be a Tensor of Tin.
    scales: A `Tensor` of type `float32`.
      The float value(s) to use as scale(s) to quantize `input`.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (input.dim_size(quantization_axis),) (per-axis quantization).
    zero_points: A `Tensor` of type `int32`.
      The int32 value(s) to use as zero_point(s) to quantize `input`.
      Same shape condition as scales.
    Tout: A `tf.DType` from: `tf.qint8, tf.qint32`.
      The type of output Tensor. A tf.DType from: tf.float32
    quantization_min_val: An `int`.
      The quantization min value to quantize `input`.
      The purpose of this attribute is typically (but not limited to) to indicate narrow range, where this is set to:
      `(Tin lowest) + 1` if narrow range, and `(Tin lowest)` otherwise.
      For example, if Tin is qint8, this is set to -127 if narrow range quantized or -128 if not.
    quantization_max_val: An `int`.
      The quantization max value to quantize `input`.
      The purpose of this attribute is typically (but not limited to) indicate narrow range, where this is set to:
      `(Tout max)` for both narrow range and not narrow range.
      For example, if Tin is qint8, this is set to 127.
    quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization. Otherwise, it must be set within range [0, input.dims()).
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantize", name, input, scales, zero_points, "Tout",
        Tout, "quantization_axis", quantization_axis, "quantization_min_val",
        quantization_min_val, "quantization_max_val", quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantize_eager_fallback(
          input, scales, zero_points, Tout=Tout,
          quantization_axis=quantization_axis,
          quantization_min_val=quantization_min_val,
          quantization_max_val=quantization_max_val, name=name, ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  quantization_min_val = _execute.make_int(quantization_min_val, "quantization_min_val")
  quantization_max_val = _execute.make_int(quantization_max_val, "quantization_max_val")
  if quantization_axis is None:
    quantization_axis = -1
  quantization_axis = _execute.make_int(quantization_axis, "quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantize", input=input, scales=scales,
                           zero_points=zero_points, Tout=Tout,
                           quantization_min_val=quantization_min_val,
                           quantization_max_val=quantization_max_val,
                           quantization_axis=quantization_axis, name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tin", _op._get_attr_type("Tin"), "Tout",
              _op._get_attr_type("Tout"), "quantization_axis",
              _op._get_attr_int("quantization_axis"), "quantization_min_val",
              _op._get_attr_int("quantization_min_val"),
              "quantization_max_val",
              _op._get_attr_int("quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantize", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantize = tf_export("raw_ops.UniformQuantize")(_ops.to_raw_op(uniform_quantize))


def uniform_quantize_eager_fallback(input, scales, zero_points, Tout, quantization_min_val, quantization_max_val, quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  quantization_min_val = _execute.make_int(quantization_min_val, "quantization_min_val")
  quantization_max_val = _execute.make_int(quantization_max_val, "quantization_max_val")
  if quantization_axis is None:
    quantization_axis = -1
  quantization_axis = _execute.make_int(quantization_axis, "quantization_axis")
  _attr_Tin, (input,) = _execute.args_to_matching_eager([input], ctx, [_dtypes.float32, ])
  scales = _ops.convert_to_tensor(scales, _dtypes.float32)
  zero_points = _ops.convert_to_tensor(zero_points, _dtypes.int32)
  _inputs_flat = [input, scales, zero_points]
  _attrs = ("Tin", _attr_Tin, "Tout", Tout, "quantization_axis",
  quantization_axis, "quantization_min_val", quantization_min_val,
  "quantization_max_val", quantization_max_val)
  _result = _execute.execute(b"UniformQuantize", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=ctx, name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantize", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantized_add(lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points, lhs_quantization_min_val, lhs_quantization_max_val, rhs_quantization_min_val, rhs_quantization_max_val, output_quantization_min_val, output_quantization_max_val, lhs_quantization_axis=-1, rhs_quantization_axis=-1, output_quantization_axis=-1, name=None):
  r"""Perform quantized add of quantized Tensor `lhs` and quantized Tensor `rhs` to make quantized `output`.

  Given quantized `lhs` and quantized `rhs`, performs quantized add on `lhs` and `rhs` to make quantized `output`.

  `UniformQuantizedAdd` follows Numpy broadcasting rules.
  The two input array shapes are compared element-wise.
  Starting with the trailing dimensions, the two dimensions either have to be equal or one of them needs to be 1.

  `lhs` and `rhs` must be quantized Tensor, where data value is quantized using the formula:
  ```
  quantized_data = clip(original_data / scale + zero_point, quantization_min_val, quantization_max_val)
  ```
  `output` is also quantized, using the same formula.

  If `lhs` and `output` is both per-axis quantized, the quantization axis must match.
  Also, if `rhs` and `output` is both per-axis quantized, the quantization axis must match.
  *Match* means the axis must match when adding, regarding the broadcasting.
  i.e. For both operands `lhs` and `rhs`,
  if `operand.quantization_axis` >= 0 and `output.quantization_axis` >= 0,
  `operand.dims` - `operand.quantization_axis` must be equal to `output.dims` - `output.quantization_axis`.

  Args:
    lhs: A `Tensor`. Must be one of the following types: `qint32`.
      Must be a quantized tensor.
    rhs: A `Tensor`. Must have the same type as `lhs`.
      Must be a quantized tensor.
    lhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale factors when quantizing the original data that `lhs` represents.
    lhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero points when quantizing original data that `lhs` represents.
      Must have same shape with `lhs_scales`.
    rhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale factors when quantizing the original data that `rhs` represents.
    rhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero points when quantizing original data that `rhs` represents.
      Must have same shape with `rhs_scales`.
    output_scales: A `Tensor` of type `float32`.
      The float value(s) to use as scale factors when quantizing original data that `output` represents.
    output_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero points when quantizing original data that output represents.
      Must have same shape with `output_scales`.
    lhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in `lhs`.
      For example, if `Tin` is `qint8`, this must be set to -127 if narrow range quantized or -128 if not.
    lhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in `lhs`.
      For example, if `Tin` is `qint8`, this must be set to 127.
    rhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in `rhs`.
      For example, if `Tin` is `qint8`, this must be set to -127 if narrow range quantized or -128 if not.
    rhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in `rhs`.
      For example, if `Tin` is `qint8`, this must be set to 127.
    output_quantization_min_val: An `int`.
      The min value of the quantized data stored in `output`.
      For example, if  `Tout` is `qint8`, this must be set to -127 if narrow range quantized or -128 if not.
    output_quantization_max_val: An `int`.
      The max value of the quantized data stored in `output`.
      For example, if `Tout` is `qint8`, this must be set to 127.
    lhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `lhs`, only per-tensor quantization is supported.
      Thus, this must be set to -1.
      Other values will raise error at OpKernel construction.
    rhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `rhs`, only per-tensor quantization
      or per-channel quantization along `kernel_output_feature_dimension` is supported.
      Thus, this must be set to -1 or `dimension_numbers.kernel_output_feature_dimension`.
      Other values will raise error at OpKernel construction.
    output_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `output`, only per-tensor quantization or per-channel quantization along `output_feature_dimension` is supported.
      Thus, this must be set to -1 or `dimension_numbers.output_feature_dimension`.
      Other values will raise error at OpKernel construction.
    name: A name for the operation (optional).

  Returns:
    A `Tensor`. Has the same type as `lhs`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantizedAdd", name, lhs, rhs, lhs_scales,
        lhs_zero_points, rhs_scales, rhs_zero_points, output_scales,
        output_zero_points, "lhs_quantization_axis", lhs_quantization_axis,
        "lhs_quantization_min_val", lhs_quantization_min_val,
        "lhs_quantization_max_val", lhs_quantization_max_val,
        "rhs_quantization_axis", rhs_quantization_axis,
        "rhs_quantization_min_val", rhs_quantization_min_val,
        "rhs_quantization_max_val", rhs_quantization_max_val,
        "output_quantization_axis", output_quantization_axis,
        "output_quantization_min_val", output_quantization_min_val,
        "output_quantization_max_val", output_quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantized_add_eager_fallback(
          lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points,
          output_scales, output_zero_points,
          lhs_quantization_axis=lhs_quantization_axis,
          lhs_quantization_min_val=lhs_quantization_min_val,
          lhs_quantization_max_val=lhs_quantization_max_val,
          rhs_quantization_axis=rhs_quantization_axis,
          rhs_quantization_min_val=rhs_quantization_min_val,
          rhs_quantization_max_val=rhs_quantization_max_val,
          output_quantization_axis=output_quantization_axis,
          output_quantization_min_val=output_quantization_min_val,
          output_quantization_max_val=output_quantization_max_val, name=name,
          ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  lhs_quantization_min_val = _execute.make_int(lhs_quantization_min_val, "lhs_quantization_min_val")
  lhs_quantization_max_val = _execute.make_int(lhs_quantization_max_val, "lhs_quantization_max_val")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if lhs_quantization_axis is None:
    lhs_quantization_axis = -1
  lhs_quantization_axis = _execute.make_int(lhs_quantization_axis, "lhs_quantization_axis")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantizedAdd", lhs=lhs, rhs=rhs, lhs_scales=lhs_scales,
                               lhs_zero_points=lhs_zero_points,
                               rhs_scales=rhs_scales,
                               rhs_zero_points=rhs_zero_points,
                               output_scales=output_scales,
                               output_zero_points=output_zero_points,
                               lhs_quantization_min_val=lhs_quantization_min_val,
                               lhs_quantization_max_val=lhs_quantization_max_val,
                               rhs_quantization_min_val=rhs_quantization_min_val,
                               rhs_quantization_max_val=rhs_quantization_max_val,
                               output_quantization_min_val=output_quantization_min_val,
                               output_quantization_max_val=output_quantization_max_val,
                               lhs_quantization_axis=lhs_quantization_axis,
                               rhs_quantization_axis=rhs_quantization_axis,
                               output_quantization_axis=output_quantization_axis,
                               name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("lhs_quantization_axis",
              _op._get_attr_int("lhs_quantization_axis"),
              "lhs_quantization_min_val",
              _op._get_attr_int("lhs_quantization_min_val"),
              "lhs_quantization_max_val",
              _op._get_attr_int("lhs_quantization_max_val"),
              "rhs_quantization_axis",
              _op._get_attr_int("rhs_quantization_axis"),
              "rhs_quantization_min_val",
              _op._get_attr_int("rhs_quantization_min_val"),
              "rhs_quantization_max_val",
              _op._get_attr_int("rhs_quantization_max_val"),
              "output_quantization_axis",
              _op._get_attr_int("output_quantization_axis"),
              "output_quantization_min_val",
              _op._get_attr_int("output_quantization_min_val"),
              "output_quantization_max_val",
              _op._get_attr_int("output_quantization_max_val"), "T",
              _op._get_attr_type("T"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantizedAdd", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantizedAdd = tf_export("raw_ops.UniformQuantizedAdd")(_ops.to_raw_op(uniform_quantized_add))


def uniform_quantized_add_eager_fallback(lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points, lhs_quantization_min_val, lhs_quantization_max_val, rhs_quantization_min_val, rhs_quantization_max_val, output_quantization_min_val, output_quantization_max_val, lhs_quantization_axis, rhs_quantization_axis, output_quantization_axis, name, ctx):
  lhs_quantization_min_val = _execute.make_int(lhs_quantization_min_val, "lhs_quantization_min_val")
  lhs_quantization_max_val = _execute.make_int(lhs_quantization_max_val, "lhs_quantization_max_val")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if lhs_quantization_axis is None:
    lhs_quantization_axis = -1
  lhs_quantization_axis = _execute.make_int(lhs_quantization_axis, "lhs_quantization_axis")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _attr_T, _inputs_T = _execute.args_to_matching_eager([lhs, rhs], ctx, [_dtypes.qint32, ])
  (lhs, rhs) = _inputs_T
  lhs_scales = _ops.convert_to_tensor(lhs_scales, _dtypes.float32)
  lhs_zero_points = _ops.convert_to_tensor(lhs_zero_points, _dtypes.int32)
  rhs_scales = _ops.convert_to_tensor(rhs_scales, _dtypes.float32)
  rhs_zero_points = _ops.convert_to_tensor(rhs_zero_points, _dtypes.int32)
  output_scales = _ops.convert_to_tensor(output_scales, _dtypes.float32)
  output_zero_points = _ops.convert_to_tensor(output_zero_points, _dtypes.int32)
  _inputs_flat = [lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points]
  _attrs = ("lhs_quantization_axis", lhs_quantization_axis,
  "lhs_quantization_min_val", lhs_quantization_min_val,
  "lhs_quantization_max_val", lhs_quantization_max_val,
  "rhs_quantization_axis", rhs_quantization_axis, "rhs_quantization_min_val",
  rhs_quantization_min_val, "rhs_quantization_max_val",
  rhs_quantization_max_val, "output_quantization_axis",
  output_quantization_axis, "output_quantization_min_val",
  output_quantization_min_val, "output_quantization_max_val",
  output_quantization_max_val, "T", _attr_T)
  _result = _execute.execute(b"UniformQuantizedAdd", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=ctx, name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantizedAdd", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantized_clip_by_value(operand, min, max, scales, zero_points, quantization_min_val, quantization_max_val, quantization_axis=-1, name=None):
  r"""Perform clip by value on the quantized Tensor `operand`.

  Given quantized `operand` which was quantized using `scales` and `zero_points`, performs clip by value using `min` and `max` values.
  If quantization_axis is -1 (per-tensor quantized), the entire operand is clipped using scalar min, max.
  Otherwise (per-channel quantized), the clipping is also done per-channel.

  Args:
    operand: A `Tensor`. Must be one of the following types: `qint32`.
      Must be a Tensor of T.
    min: A `Tensor`. Must have the same type as `operand`.
      The min value(s) to clip operand. Must be a Tensor of T.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (operand.dim_size(quantization_axis),) (per-axis quantization).
    max: A `Tensor`. Must have the same type as `operand`.
      The min value(s) to clip operand. Must be a Tensor of T.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (operand.dim_size(quantization_axis),) (per-axis quantization).
    scales: A `Tensor` of type `float32`.
      The float value(s) used as scale(s) when quantizing `operand`, `min` and `max`.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (operand.dim_size(quantization_axis),) (per-axis quantization).
    zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point(s) when quantizing `operand`, `min` and `max`.
      Same shape condition as scales.
    quantization_min_val: An `int`.
      The quantization min value that was used when operand was quantized.
    quantization_max_val: An `int`.
      The quantization max value that was used when operand was quantized.
    quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization. Otherwise, it must be set within range [0, operand.dims()).
    name: A name for the operation (optional).

  Returns:
    A `Tensor`. Has the same type as `operand`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantizedClipByValue", name, operand, min, max, scales,
        zero_points, "quantization_axis", quantization_axis,
        "quantization_min_val", quantization_min_val, "quantization_max_val",
        quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantized_clip_by_value_eager_fallback(
          operand, min, max, scales, zero_points,
          quantization_axis=quantization_axis,
          quantization_min_val=quantization_min_val,
          quantization_max_val=quantization_max_val, name=name, ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  quantization_min_val = _execute.make_int(quantization_min_val, "quantization_min_val")
  quantization_max_val = _execute.make_int(quantization_max_val, "quantization_max_val")
  if quantization_axis is None:
    quantization_axis = -1
  quantization_axis = _execute.make_int(quantization_axis, "quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantizedClipByValue", operand=operand, min=min, max=max,
                                       scales=scales, zero_points=zero_points,
                                       quantization_min_val=quantization_min_val,
                                       quantization_max_val=quantization_max_val,
                                       quantization_axis=quantization_axis,
                                       name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("T", _op._get_attr_type("T"), "quantization_axis",
              _op._get_attr_int("quantization_axis"), "quantization_min_val",
              _op._get_attr_int("quantization_min_val"),
              "quantization_max_val",
              _op._get_attr_int("quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantizedClipByValue", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantizedClipByValue = tf_export("raw_ops.UniformQuantizedClipByValue")(_ops.to_raw_op(uniform_quantized_clip_by_value))


def uniform_quantized_clip_by_value_eager_fallback(operand, min, max, scales, zero_points, quantization_min_val, quantization_max_val, quantization_axis, name, ctx):
  quantization_min_val = _execute.make_int(quantization_min_val, "quantization_min_val")
  quantization_max_val = _execute.make_int(quantization_max_val, "quantization_max_val")
  if quantization_axis is None:
    quantization_axis = -1
  quantization_axis = _execute.make_int(quantization_axis, "quantization_axis")
  _attr_T, _inputs_T = _execute.args_to_matching_eager([operand, min, max], ctx, [_dtypes.qint32, ])
  (operand, min, max) = _inputs_T
  scales = _ops.convert_to_tensor(scales, _dtypes.float32)
  zero_points = _ops.convert_to_tensor(zero_points, _dtypes.int32)
  _inputs_flat = [operand, min, max, scales, zero_points]
  _attrs = ("T", _attr_T, "quantization_axis", quantization_axis,
  "quantization_min_val", quantization_min_val, "quantization_max_val",
  quantization_max_val)
  _result = _execute.execute(b"UniformQuantizedClipByValue", 1,
                             inputs=_inputs_flat, attrs=_attrs, ctx=ctx,
                             name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantizedClipByValue", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantized_convolution(lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points, Tout, padding, lhs_quantization_min_val, lhs_quantization_max_val, rhs_quantization_min_val, rhs_quantization_max_val, output_quantization_min_val, output_quantization_max_val, window_strides=[], explicit_padding=[], lhs_dilation=[], rhs_dilation=[], batch_group_count=1, feature_group_count=1, dimension_numbers="", lhs_quantization_axis=-1, rhs_quantization_axis=-1, output_quantization_axis=-1, name=None):
  r"""Perform quantized convolution of quantized Tensor `lhs` and quantized Tensor `rhs`. to make quantized `output`.

  Given quantized `lhs` and quantized `rhs`, performs quantized dot on `lhs` and `rhs` to make quantized `output`.

  `lhs` and `rhs` must be Tensors of same rank, and meet following shape conditions.
  - `lhs_feature` % `feature_group_count` == 0
  - `lhs_feature` % `rhs_input_feature` == 0
  - `lhs_feature` / `feature_group_count` == `rhs_input_feature`
  - `rhs_output_feature` % `feature_group_count` == 0
  - `lhs_batch` % `batch_group_count` == 0
  - `rhs_output_feature` % `batch_group_count` == 0

  `lhs` and `rhs` must be quantized Tensor, where data value is quantized using the formula:
  ```
  quantized_data = clip(original_data / scale + zero_point, quantization_min_val, quantization_max_val)
  ```
  `output` is also quantized, using the same formula.
  If `rhs` is per-tensor quantized, `output` must be also per-tensor quantized.

  Args:
    lhs: A `Tensor`. Must be one of the following types: `qint8`.
      Must be a quantized tensor, rank >= 3.
    rhs: A `Tensor`. Must have the same type as `lhs`.
      Must be a quantized tensor, same rank as `lhs`.
    lhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale factors when quantizing the original data that `lhs` represents.
      Must be a scalar `Tensor` (`lhs` supports only per-tensor quantization).
    lhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero points when quantizing original data that `lhs` represents.
      Same shape condition as `lhs_scales`.
    rhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale factors when quantizing the original data that `rhs` represents.
      Must be a scalar `Tensor` for per-tensor quantization,
      or 1D `Tensor` of size `rhs.dim_size(kernel_output_feature_dimension)`, for per-channel quantization.
    rhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero points when quantizing original data that `rhs` represents.
      Same shape condition as `rhs_scales`.
    output_scales: A `Tensor` of type `float32`.
      The float value(s) to use as scale factors when quantizing original data that `output` represents.
      Must be a scalar `Tensor` for per-tensor quantization,
      or 1D `Tensor` of size `rhs.dim_size(kernel_output_feature_dimension)`
      - which is equal to `output.dim_size(output_feature_dimension)`,
      for per-channel quantization.
      If `rhs` is per-tensor quantized, output must be also per-tensor quantized.
      This means that if `rhs_scales` and `rhs_zero_points` are scalar `Tensor`s, `output_scales` and `output_zero_points` must be scalar `Tensor`s as well.
    output_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero points when quantizing original data that output represents.
      Same shape condition as `output_scales`.
    Tout: A `tf.DType` from: `tf.qint32`. The type of `output` `Tensor`.
    padding: A `string`.
      string from: `"SAME"`, `"VALID"`, or `"EXPLICIT"`, indicating the type of padding algorithm to use.
    lhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in `lhs`.
      For example, if `Tin` is `qint8`, this must be set to -127 if narrow range quantized or -128 if not.
    lhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in `lhs`.
      For example, if `Tin` is `qint8`, this must be set to 127.
    rhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in `rhs`.
      For example, if `Tin` is `qint8`, this must be set to -127 if narrow range quantized or -128 if not.
    rhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in `rhs`.
      For example, if `Tin` is `qint8`, this must be set to 127.
    output_quantization_min_val: An `int`.
      The min value of the quantized data stored in `output`.
      For example, if  `Tout` is `qint8`, this must be set to -127 if narrow range quantized or -128 if not.
    output_quantization_max_val: An `int`.
      The max value of the quantized data stored in `output`.
      For example, if `Tout` is `qint8`, this must be set to 127.
    window_strides: An optional list of `ints`. Defaults to `[]`.
      The stride of the sliding window for each spatial dimension of `lhs`.
      Must be an empty list (default) or a list of size (number of spatial dimensions).
      If an empty list is provided, the stride for each spatial dimension is set to 1.
    explicit_padding: An optional list of `ints`. Defaults to `[]`.
      If `padding` is `"EXPLICIT"`, must be set as a list indicating
      the explicit paddings at the start and end of each `lhs` spatial dimension.
      Otherwise, this must be empty.

      (If used,) Must be a list of size `2 * (number of lhs spatial dimensions)`,
      where `(explicit_padding[2 * i], explicit_padding[2 * i + 1])` indicates
      `(start_padding, end_padding)` of `spatial_dimensions[i]`.
    lhs_dilation: An optional list of `ints`. Defaults to `[]`.
      The dilation factor to apply in each spatial dimension of `lhs`.
      Must be an empty list (default) or a list of size (number of `lhs` spatial dimensions).
      If empty list, the dilation for each `lhs` spatial dimension is set to 1.
    rhs_dilation: An optional list of `ints`. Defaults to `[]`.
      The dilation factor to apply in each spatial dimension of `rhs`.
      Must be an empty list (default) or a list of size (number of `rhs` spatial dimensions).
      If empty list, the dilation for each `rhs` spatial dimension is set to 1.
    batch_group_count: An optional `int`. Defaults to `1`.
      The number of batch groups. Used for grouped filters.
      Must be a divisor of `output_feature`.
    feature_group_count: An optional `int`. Defaults to `1`.
      The number of feature groups. Used for grouped convolutions.
      Must be a divisor of both `lhs_feature` and `output_feature`.
    dimension_numbers: An optional `string`. Defaults to `""`.
      Structure of dimension information for the convolution op.
      Must be an empty string (default) or a serialized string of `tensorflow.UniformQuantizedConvolutionDimensionNumbersAttr` proto.
      If empty string, the default is `("NCHW", "OIHW", "NCHW")` (for a 2D convolution).
    lhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `lhs`, only per-tensor quantization is supported.
      Thus, this must be set to -1.
      Other values will raise error at OpKernel construction.
    rhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `rhs`, only per-tensor quantization
      or per-channel quantization along `kernel_output_feature_dimension` is supported.
      Thus, this must be set to -1 or `dimension_numbers.kernel_output_feature_dimension`.
      Other values will raise error at OpKernel construction.
    output_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `output`, only per-tensor quantization or per-channel quantization along `output_feature_dimension` is supported.
      Thus, this must be set to -1 or `dimension_numbers.output_feature_dimension`.
      Other values will raise error at OpKernel construction.
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantizedConvolution", name, lhs, rhs, lhs_scales,
        lhs_zero_points, rhs_scales, rhs_zero_points, output_scales,
        output_zero_points, "Tout", Tout, "window_strides", window_strides,
        "padding", padding, "explicit_padding", explicit_padding,
        "lhs_dilation", lhs_dilation, "rhs_dilation", rhs_dilation,
        "batch_group_count", batch_group_count, "feature_group_count",
        feature_group_count, "dimension_numbers", dimension_numbers,
        "lhs_quantization_axis", lhs_quantization_axis,
        "lhs_quantization_min_val", lhs_quantization_min_val,
        "lhs_quantization_max_val", lhs_quantization_max_val,
        "rhs_quantization_axis", rhs_quantization_axis,
        "rhs_quantization_min_val", rhs_quantization_min_val,
        "rhs_quantization_max_val", rhs_quantization_max_val,
        "output_quantization_axis", output_quantization_axis,
        "output_quantization_min_val", output_quantization_min_val,
        "output_quantization_max_val", output_quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantized_convolution_eager_fallback(
          lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points,
          output_scales, output_zero_points, Tout=Tout,
          window_strides=window_strides, padding=padding,
          explicit_padding=explicit_padding, lhs_dilation=lhs_dilation,
          rhs_dilation=rhs_dilation, batch_group_count=batch_group_count,
          feature_group_count=feature_group_count,
          dimension_numbers=dimension_numbers,
          lhs_quantization_axis=lhs_quantization_axis,
          lhs_quantization_min_val=lhs_quantization_min_val,
          lhs_quantization_max_val=lhs_quantization_max_val,
          rhs_quantization_axis=rhs_quantization_axis,
          rhs_quantization_min_val=rhs_quantization_min_val,
          rhs_quantization_max_val=rhs_quantization_max_val,
          output_quantization_axis=output_quantization_axis,
          output_quantization_min_val=output_quantization_min_val,
          output_quantization_max_val=output_quantization_max_val, name=name,
          ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  padding = _execute.make_str(padding, "padding")
  lhs_quantization_min_val = _execute.make_int(lhs_quantization_min_val, "lhs_quantization_min_val")
  lhs_quantization_max_val = _execute.make_int(lhs_quantization_max_val, "lhs_quantization_max_val")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if window_strides is None:
    window_strides = []
  if not isinstance(window_strides, (list, tuple)):
    raise TypeError(
        "Expected list for 'window_strides' argument to "
        "'uniform_quantized_convolution' Op, not %r." % window_strides)
  window_strides = [_execute.make_int(_i, "window_strides") for _i in window_strides]
  if explicit_padding is None:
    explicit_padding = []
  if not isinstance(explicit_padding, (list, tuple)):
    raise TypeError(
        "Expected list for 'explicit_padding' argument to "
        "'uniform_quantized_convolution' Op, not %r." % explicit_padding)
  explicit_padding = [_execute.make_int(_i, "explicit_padding") for _i in explicit_padding]
  if lhs_dilation is None:
    lhs_dilation = []
  if not isinstance(lhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'lhs_dilation' argument to "
        "'uniform_quantized_convolution' Op, not %r." % lhs_dilation)
  lhs_dilation = [_execute.make_int(_i, "lhs_dilation") for _i in lhs_dilation]
  if rhs_dilation is None:
    rhs_dilation = []
  if not isinstance(rhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'rhs_dilation' argument to "
        "'uniform_quantized_convolution' Op, not %r." % rhs_dilation)
  rhs_dilation = [_execute.make_int(_i, "rhs_dilation") for _i in rhs_dilation]
  if batch_group_count is None:
    batch_group_count = 1
  batch_group_count = _execute.make_int(batch_group_count, "batch_group_count")
  if feature_group_count is None:
    feature_group_count = 1
  feature_group_count = _execute.make_int(feature_group_count, "feature_group_count")
  if dimension_numbers is None:
    dimension_numbers = ""
  dimension_numbers = _execute.make_str(dimension_numbers, "dimension_numbers")
  if lhs_quantization_axis is None:
    lhs_quantization_axis = -1
  lhs_quantization_axis = _execute.make_int(lhs_quantization_axis, "lhs_quantization_axis")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantizedConvolution", lhs=lhs, rhs=rhs,
                                       lhs_scales=lhs_scales,
                                       lhs_zero_points=lhs_zero_points,
                                       rhs_scales=rhs_scales,
                                       rhs_zero_points=rhs_zero_points,
                                       output_scales=output_scales,
                                       output_zero_points=output_zero_points,
                                       Tout=Tout, padding=padding,
                                       lhs_quantization_min_val=lhs_quantization_min_val,
                                       lhs_quantization_max_val=lhs_quantization_max_val,
                                       rhs_quantization_min_val=rhs_quantization_min_val,
                                       rhs_quantization_max_val=rhs_quantization_max_val,
                                       output_quantization_min_val=output_quantization_min_val,
                                       output_quantization_max_val=output_quantization_max_val,
                                       window_strides=window_strides,
                                       explicit_padding=explicit_padding,
                                       lhs_dilation=lhs_dilation,
                                       rhs_dilation=rhs_dilation,
                                       batch_group_count=batch_group_count,
                                       feature_group_count=feature_group_count,
                                       dimension_numbers=dimension_numbers,
                                       lhs_quantization_axis=lhs_quantization_axis,
                                       rhs_quantization_axis=rhs_quantization_axis,
                                       output_quantization_axis=output_quantization_axis,
                                       name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tin", _op._get_attr_type("Tin"), "Tout",
              _op._get_attr_type("Tout"), "window_strides",
              _op.get_attr("window_strides"), "padding",
              _op.get_attr("padding"), "explicit_padding",
              _op.get_attr("explicit_padding"), "lhs_dilation",
              _op.get_attr("lhs_dilation"), "rhs_dilation",
              _op.get_attr("rhs_dilation"), "batch_group_count",
              _op._get_attr_int("batch_group_count"), "feature_group_count",
              _op._get_attr_int("feature_group_count"), "dimension_numbers",
              _op.get_attr("dimension_numbers"), "lhs_quantization_axis",
              _op._get_attr_int("lhs_quantization_axis"),
              "lhs_quantization_min_val",
              _op._get_attr_int("lhs_quantization_min_val"),
              "lhs_quantization_max_val",
              _op._get_attr_int("lhs_quantization_max_val"),
              "rhs_quantization_axis",
              _op._get_attr_int("rhs_quantization_axis"),
              "rhs_quantization_min_val",
              _op._get_attr_int("rhs_quantization_min_val"),
              "rhs_quantization_max_val",
              _op._get_attr_int("rhs_quantization_max_val"),
              "output_quantization_axis",
              _op._get_attr_int("output_quantization_axis"),
              "output_quantization_min_val",
              _op._get_attr_int("output_quantization_min_val"),
              "output_quantization_max_val",
              _op._get_attr_int("output_quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantizedConvolution", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantizedConvolution = tf_export("raw_ops.UniformQuantizedConvolution")(_ops.to_raw_op(uniform_quantized_convolution))


def uniform_quantized_convolution_eager_fallback(lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points, Tout, padding, lhs_quantization_min_val, lhs_quantization_max_val, rhs_quantization_min_val, rhs_quantization_max_val, output_quantization_min_val, output_quantization_max_val, window_strides, explicit_padding, lhs_dilation, rhs_dilation, batch_group_count, feature_group_count, dimension_numbers, lhs_quantization_axis, rhs_quantization_axis, output_quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  padding = _execute.make_str(padding, "padding")
  lhs_quantization_min_val = _execute.make_int(lhs_quantization_min_val, "lhs_quantization_min_val")
  lhs_quantization_max_val = _execute.make_int(lhs_quantization_max_val, "lhs_quantization_max_val")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if window_strides is None:
    window_strides = []
  if not isinstance(window_strides, (list, tuple)):
    raise TypeError(
        "Expected list for 'window_strides' argument to "
        "'uniform_quantized_convolution' Op, not %r." % window_strides)
  window_strides = [_execute.make_int(_i, "window_strides") for _i in window_strides]
  if explicit_padding is None:
    explicit_padding = []
  if not isinstance(explicit_padding, (list, tuple)):
    raise TypeError(
        "Expected list for 'explicit_padding' argument to "
        "'uniform_quantized_convolution' Op, not %r." % explicit_padding)
  explicit_padding = [_execute.make_int(_i, "explicit_padding") for _i in explicit_padding]
  if lhs_dilation is None:
    lhs_dilation = []
  if not isinstance(lhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'lhs_dilation' argument to "
        "'uniform_quantized_convolution' Op, not %r." % lhs_dilation)
  lhs_dilation = [_execute.make_int(_i, "lhs_dilation") for _i in lhs_dilation]
  if rhs_dilation is None:
    rhs_dilation = []
  if not isinstance(rhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'rhs_dilation' argument to "
        "'uniform_quantized_convolution' Op, not %r." % rhs_dilation)
  rhs_dilation = [_execute.make_int(_i, "rhs_dilation") for _i in rhs_dilation]
  if batch_group_count is None:
    batch_group_count = 1
  batch_group_count = _execute.make_int(batch_group_count, "batch_group_count")
  if feature_group_count is None:
    feature_group_count = 1
  feature_group_count = _execute.make_int(feature_group_count, "feature_group_count")
  if dimension_numbers is None:
    dimension_numbers = ""
  dimension_numbers = _execute.make_str(dimension_numbers, "dimension_numbers")
  if lhs_quantization_axis is None:
    lhs_quantization_axis = -1
  lhs_quantization_axis = _execute.make_int(lhs_quantization_axis, "lhs_quantization_axis")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _attr_Tin, _inputs_Tin = _execute.args_to_matching_eager([lhs, rhs], ctx, [_dtypes.qint8, ])
  (lhs, rhs) = _inputs_Tin
  lhs_scales = _ops.convert_to_tensor(lhs_scales, _dtypes.float32)
  lhs_zero_points = _ops.convert_to_tensor(lhs_zero_points, _dtypes.int32)
  rhs_scales = _ops.convert_to_tensor(rhs_scales, _dtypes.float32)
  rhs_zero_points = _ops.convert_to_tensor(rhs_zero_points, _dtypes.int32)
  output_scales = _ops.convert_to_tensor(output_scales, _dtypes.float32)
  output_zero_points = _ops.convert_to_tensor(output_zero_points, _dtypes.int32)
  _inputs_flat = [lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points]
  _attrs = ("Tin", _attr_Tin, "Tout", Tout, "window_strides", window_strides,
  "padding", padding, "explicit_padding", explicit_padding, "lhs_dilation",
  lhs_dilation, "rhs_dilation", rhs_dilation, "batch_group_count",
  batch_group_count, "feature_group_count", feature_group_count,
  "dimension_numbers", dimension_numbers, "lhs_quantization_axis",
  lhs_quantization_axis, "lhs_quantization_min_val", lhs_quantization_min_val,
  "lhs_quantization_max_val", lhs_quantization_max_val,
  "rhs_quantization_axis", rhs_quantization_axis, "rhs_quantization_min_val",
  rhs_quantization_min_val, "rhs_quantization_max_val",
  rhs_quantization_max_val, "output_quantization_axis",
  output_quantization_axis, "output_quantization_min_val",
  output_quantization_min_val, "output_quantization_max_val",
  output_quantization_max_val)
  _result = _execute.execute(b"UniformQuantizedConvolution", 1,
                             inputs=_inputs_flat, attrs=_attrs, ctx=ctx,
                             name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantizedConvolution", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantized_convolution_hybrid(lhs, rhs, rhs_scales, rhs_zero_points, Tout, padding, rhs_quantization_min_val, rhs_quantization_max_val, window_strides=[], explicit_padding=[], lhs_dilation=[], rhs_dilation=[], batch_group_count=1, feature_group_count=1, dimension_numbers="", rhs_quantization_axis=-1, name=None):
  r"""Perform hybrid quantized convolution of float Tensor `lhs` and quantized Tensor `rhs`.

  Given float `lhs` and quantized `rhs`, internally performs quantization on `lhs`,
  and then performs quantized convolution on quantized `lhs` and `rhs`.

  The internal quantization on `lhs` is a quantization to `Trhs`, dynamic range,
  per-batch (per-axis along axis `dimension_numbers.input_batch_dimension`), asymmetric,
  and not narrow range (the range is [Trhs_MIN, Trhs_MAX]).

  `lhs` and `rhs` must be Tensors of same rank, and meet following shape conditions.
  - lhs_feature % feature_group_count == 0
  - lhs_feature % rhs_input_feature == 0
  - lhs_feature / feature_group_count == rhs_input_feature
  - rhs_output_feature % feature_group_count == 0
  - lhs_batch % batch_group_count == 0
  - rhs_output_feature % batch_group_count == 0

  `rhs` must be quantized Tensor, where its data value is quantized using the formula:
  quantized_data = clip(original_data / scale + zero_point, quantization_min_val, quantization_max_val).

  Args:
    lhs: A `Tensor`. Must be one of the following types: `float32`.
      Must be a non-quantized Tensor of `Tlhs`, rank >= 3.
    rhs: A `Tensor`. Must be one of the following types: `qint8`.
      Must be a quantized Tensor of `Trhs`, same rank as `lhs`.
    rhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale factors when quantizing the original data that `rhs` represents.
      Must be a scalar Tensor for per-tensor quantization,
      or 1D Tensor of size `rhs.dim_size(kernel_output_feature_dimension)`, for per-channel quantization.
    rhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point when quantizing original data that `rhs` represents.
      Same shape condition as `rhs_scales`.
    Tout: A `tf.DType` from: `tf.float32`. The type of output Tensor.
    padding: A `string`.
      string from: `"SAME"`, `"VALID"`, or `"EXPLICIT"`, indicating the type of padding algorithm to use.
    rhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in `rhs`.
      For example, if `Trhs` is qint8, this must be set to -127 if narrow range quantized or -128 if not.
    rhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in `rhs`.
      For example, if `Trhs` is qint8, this must be set to 127.
    window_strides: An optional list of `ints`. Defaults to `[]`.
      The stride of the sliding window for each spatial dimension of `lhs`.
      Must be an empty list (default) or a list of size (number of spatial dimensions).
      If an empty list is provided, the stride for each spatial dimension is set to 1.
    explicit_padding: An optional list of `ints`. Defaults to `[]`.
      If `padding` Attr is `"EXPLICIT"`, must be set as a list indicating
      the explicit paddings at the start and end of each lhs spatial dimension.
      Otherwise, this Attr is must be empty.

      (If used,) Must be a list of size 2 * (number of lhs spatial dimensions),
      where (explicit_padding[2 * i], explicit_padding[2 * i + 1]) indicates
      spatial_dimensions[i] (start_padding, end_padding).
    lhs_dilation: An optional list of `ints`. Defaults to `[]`.
      The dilation factor to apply in each spatial dimension of `lhs`.
      Must be an empty list (default) or a list of size (number of lhs spatial dimensions).
      If empty list, the dilation for each lhs spatial dimension is set to 1.
    rhs_dilation: An optional list of `ints`. Defaults to `[]`.
      The dilation factor to apply in each spatial dimension of `rhs`.
      Must be an empty list (default) or a list of size (number of rhs spatial dimensions).
      If empty list, the dilation for each rhs spatial dimension is set to 1.
    batch_group_count: An optional `int`. Defaults to `1`.
      The number of batch groups. Used for grouped filters.
      Must be a divisor of output_feature.
    feature_group_count: An optional `int`. Defaults to `1`.
      The number of feature groups. Used for grouped convolutions.
      Must be a divisor of both lhs_feature and output_feature.
    dimension_numbers: An optional `string`. Defaults to `""`.
      Structure of dimension information for the convolution op.
      Must be an empty string (default) or a serialized string of tensorflow.UniformQuantizedConvolutionDimensionNumbersAttr proto.
      If empty string, the default is `("NCHW", "OIHW", "NCHW")` (for a 2D convolution).
    rhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For the `rhs`, only per-tensor quantization
      or per-channel quantization along kernel_output_feature_dimension is supported.
      Thus, this attribute must be set to -1 or `dimension_numbers.kernel_output_feature_dimension`.
      Other values will raise error at OpKernel construction.
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantizedConvolutionHybrid", name, lhs, rhs, rhs_scales,
        rhs_zero_points, "Tout", Tout, "window_strides", window_strides,
        "padding", padding, "explicit_padding", explicit_padding,
        "lhs_dilation", lhs_dilation, "rhs_dilation", rhs_dilation,
        "batch_group_count", batch_group_count, "feature_group_count",
        feature_group_count, "dimension_numbers", dimension_numbers,
        "rhs_quantization_axis", rhs_quantization_axis,
        "rhs_quantization_min_val", rhs_quantization_min_val,
        "rhs_quantization_max_val", rhs_quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantized_convolution_hybrid_eager_fallback(
          lhs, rhs, rhs_scales, rhs_zero_points, Tout=Tout,
          window_strides=window_strides, padding=padding,
          explicit_padding=explicit_padding, lhs_dilation=lhs_dilation,
          rhs_dilation=rhs_dilation, batch_group_count=batch_group_count,
          feature_group_count=feature_group_count,
          dimension_numbers=dimension_numbers,
          rhs_quantization_axis=rhs_quantization_axis,
          rhs_quantization_min_val=rhs_quantization_min_val,
          rhs_quantization_max_val=rhs_quantization_max_val, name=name,
          ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  padding = _execute.make_str(padding, "padding")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  if window_strides is None:
    window_strides = []
  if not isinstance(window_strides, (list, tuple)):
    raise TypeError(
        "Expected list for 'window_strides' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % window_strides)
  window_strides = [_execute.make_int(_i, "window_strides") for _i in window_strides]
  if explicit_padding is None:
    explicit_padding = []
  if not isinstance(explicit_padding, (list, tuple)):
    raise TypeError(
        "Expected list for 'explicit_padding' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % explicit_padding)
  explicit_padding = [_execute.make_int(_i, "explicit_padding") for _i in explicit_padding]
  if lhs_dilation is None:
    lhs_dilation = []
  if not isinstance(lhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'lhs_dilation' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % lhs_dilation)
  lhs_dilation = [_execute.make_int(_i, "lhs_dilation") for _i in lhs_dilation]
  if rhs_dilation is None:
    rhs_dilation = []
  if not isinstance(rhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'rhs_dilation' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % rhs_dilation)
  rhs_dilation = [_execute.make_int(_i, "rhs_dilation") for _i in rhs_dilation]
  if batch_group_count is None:
    batch_group_count = 1
  batch_group_count = _execute.make_int(batch_group_count, "batch_group_count")
  if feature_group_count is None:
    feature_group_count = 1
  feature_group_count = _execute.make_int(feature_group_count, "feature_group_count")
  if dimension_numbers is None:
    dimension_numbers = ""
  dimension_numbers = _execute.make_str(dimension_numbers, "dimension_numbers")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantizedConvolutionHybrid", lhs=lhs, rhs=rhs,
                                             rhs_scales=rhs_scales,
                                             rhs_zero_points=rhs_zero_points,
                                             Tout=Tout, padding=padding,
                                             rhs_quantization_min_val=rhs_quantization_min_val,
                                             rhs_quantization_max_val=rhs_quantization_max_val,
                                             window_strides=window_strides,
                                             explicit_padding=explicit_padding,
                                             lhs_dilation=lhs_dilation,
                                             rhs_dilation=rhs_dilation,
                                             batch_group_count=batch_group_count,
                                             feature_group_count=feature_group_count,
                                             dimension_numbers=dimension_numbers,
                                             rhs_quantization_axis=rhs_quantization_axis,
                                             name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tlhs", _op._get_attr_type("Tlhs"), "Trhs",
              _op._get_attr_type("Trhs"), "Tout", _op._get_attr_type("Tout"),
              "window_strides", _op.get_attr("window_strides"), "padding",
              _op.get_attr("padding"), "explicit_padding",
              _op.get_attr("explicit_padding"), "lhs_dilation",
              _op.get_attr("lhs_dilation"), "rhs_dilation",
              _op.get_attr("rhs_dilation"), "batch_group_count",
              _op._get_attr_int("batch_group_count"), "feature_group_count",
              _op._get_attr_int("feature_group_count"), "dimension_numbers",
              _op.get_attr("dimension_numbers"), "rhs_quantization_axis",
              _op._get_attr_int("rhs_quantization_axis"),
              "rhs_quantization_min_val",
              _op._get_attr_int("rhs_quantization_min_val"),
              "rhs_quantization_max_val",
              _op._get_attr_int("rhs_quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantizedConvolutionHybrid", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantizedConvolutionHybrid = tf_export("raw_ops.UniformQuantizedConvolutionHybrid")(_ops.to_raw_op(uniform_quantized_convolution_hybrid))


def uniform_quantized_convolution_hybrid_eager_fallback(lhs, rhs, rhs_scales, rhs_zero_points, Tout, padding, rhs_quantization_min_val, rhs_quantization_max_val, window_strides, explicit_padding, lhs_dilation, rhs_dilation, batch_group_count, feature_group_count, dimension_numbers, rhs_quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  padding = _execute.make_str(padding, "padding")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  if window_strides is None:
    window_strides = []
  if not isinstance(window_strides, (list, tuple)):
    raise TypeError(
        "Expected list for 'window_strides' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % window_strides)
  window_strides = [_execute.make_int(_i, "window_strides") for _i in window_strides]
  if explicit_padding is None:
    explicit_padding = []
  if not isinstance(explicit_padding, (list, tuple)):
    raise TypeError(
        "Expected list for 'explicit_padding' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % explicit_padding)
  explicit_padding = [_execute.make_int(_i, "explicit_padding") for _i in explicit_padding]
  if lhs_dilation is None:
    lhs_dilation = []
  if not isinstance(lhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'lhs_dilation' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % lhs_dilation)
  lhs_dilation = [_execute.make_int(_i, "lhs_dilation") for _i in lhs_dilation]
  if rhs_dilation is None:
    rhs_dilation = []
  if not isinstance(rhs_dilation, (list, tuple)):
    raise TypeError(
        "Expected list for 'rhs_dilation' argument to "
        "'uniform_quantized_convolution_hybrid' Op, not %r." % rhs_dilation)
  rhs_dilation = [_execute.make_int(_i, "rhs_dilation") for _i in rhs_dilation]
  if batch_group_count is None:
    batch_group_count = 1
  batch_group_count = _execute.make_int(batch_group_count, "batch_group_count")
  if feature_group_count is None:
    feature_group_count = 1
  feature_group_count = _execute.make_int(feature_group_count, "feature_group_count")
  if dimension_numbers is None:
    dimension_numbers = ""
  dimension_numbers = _execute.make_str(dimension_numbers, "dimension_numbers")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  _attr_Tlhs, (lhs,) = _execute.args_to_matching_eager([lhs], ctx, [_dtypes.float32, ])
  _attr_Trhs, (rhs,) = _execute.args_to_matching_eager([rhs], ctx, [_dtypes.qint8, ])
  rhs_scales = _ops.convert_to_tensor(rhs_scales, _dtypes.float32)
  rhs_zero_points = _ops.convert_to_tensor(rhs_zero_points, _dtypes.int32)
  _inputs_flat = [lhs, rhs, rhs_scales, rhs_zero_points]
  _attrs = ("Tlhs", _attr_Tlhs, "Trhs", _attr_Trhs, "Tout", Tout,
  "window_strides", window_strides, "padding", padding, "explicit_padding",
  explicit_padding, "lhs_dilation", lhs_dilation, "rhs_dilation",
  rhs_dilation, "batch_group_count", batch_group_count, "feature_group_count",
  feature_group_count, "dimension_numbers", dimension_numbers,
  "rhs_quantization_axis", rhs_quantization_axis, "rhs_quantization_min_val",
  rhs_quantization_min_val, "rhs_quantization_max_val",
  rhs_quantization_max_val)
  _result = _execute.execute(b"UniformQuantizedConvolutionHybrid", 1,
                             inputs=_inputs_flat, attrs=_attrs, ctx=ctx,
                             name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantizedConvolutionHybrid", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantized_dot(lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points, Tout, lhs_quantization_min_val, lhs_quantization_max_val, rhs_quantization_min_val, rhs_quantization_max_val, output_quantization_min_val, output_quantization_max_val, lhs_quantization_axis=-1, rhs_quantization_axis=-1, output_quantization_axis=-1, name=None):
  r"""Perform quantized dot of quantized Tensor `lhs` and quantized Tensor `rhs` to make quantized `output`.

  Given quantized `lhs` and quantized `rhs`, performs quantized dot on `lhs` and `rhs` to make quantized `output`.
  `lhs` and `rhs` must be 2D Tensors and the lhs.dim_size(1) must match rhs.dim_size(0).
  `lhs` and `rhs` must be quantized Tensor, where data value is quantized using the formula:
  quantized_data = clip(original_data / scale + zero_point, quantization_min_val, quantization_max_val).
  `output` is also quantized, using the same formula.
  If `rhs` is per-tensor quantized, `output` must be also per-tensor quantized.

  Args:
    lhs: A `Tensor`. Must be one of the following types: `qint8`.
      Must be a 2D Tensor of Tin.
    rhs: A `Tensor`. Must have the same type as `lhs`.
      Must be a 2D Tensor of Tin.
    lhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale when quantizing original data that lhs represents.
      Must be a scalar Tensor (lhs supports only per-tensor quantization).
    lhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point when quantizing original data that lhs represents.
      Same shape condition as lhs_scales.
    rhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale when quantizing original data that rhs represents.
      Must be a scalar Tensor (per-tensor quantization) or 1D Tensor of size (rhs.dim_size(1),) (per-channel quantization).
    rhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point when quantizing original data that rhs represents.
      Same shape condition as rhs_scales.
    output_scales: A `Tensor` of type `float32`.
      The float value(s) to use as scales when quantizing original data that output represents.
      Must be a scalar Tensor (per-tensor quantization) or 1D Tensor of size (output.dim_size(1),) (per-channel quantization).
      If rhs is per-tensor quantized, output must be also per-tensor quantized.
      This means that if rhs_scales and rhs_zero_points are scalar Tensors, output_scales and output_zero_points must be scalar Tensors as well.
    output_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point when quantizing original data that output represents.
      Same shape condition as rhs_scales.
    Tout: A `tf.DType` from: `tf.qint32`. The type of output Tensor.
    lhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in lhs.
      For example, if Tin is qint8, this must be set to -127 if narrow range quantized or -128 if not.
    lhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in rhs.
      For example, if Tin is qint8, this must be set to 127.
    rhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in rhs.
      For example, if Trhs is qint8, this must be set to -127 if narrow range quantized or -128 if not.
    rhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in rhs.
      For example, if Trhs is qint8, this must be set to 127.
    output_quantization_min_val: An `int`.
      The min value of the quantized data stored in output.
      For example, if Tout is qint8, this must be set to -127 if narrow range quantized or -128 if not.
    output_quantization_max_val: An `int`.
      The max value of the quantized data stored in output.
      For example, if Tout is qint8, this must be set to 127.
    lhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For dot op lhs, only per-tensor quantization is supported.
      Thus, this attribute must be set to -1. Other values are rejected.
    rhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For dot op rhs, only per-tensor quantization or per-channel quantization along dimension 1 is supported.
      Thus, this attribute must be set to -1 or 1. Other values are rejected.
    output_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For dot op output, only per-tensor quantization or per-channel quantization along dimension 1 is supported.
      Thus, this attribute must be set to -1 or 1. Other values are rejected.
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantizedDot", name, lhs, rhs, lhs_scales,
        lhs_zero_points, rhs_scales, rhs_zero_points, output_scales,
        output_zero_points, "Tout", Tout, "lhs_quantization_axis",
        lhs_quantization_axis, "lhs_quantization_min_val",
        lhs_quantization_min_val, "lhs_quantization_max_val",
        lhs_quantization_max_val, "rhs_quantization_axis",
        rhs_quantization_axis, "rhs_quantization_min_val",
        rhs_quantization_min_val, "rhs_quantization_max_val",
        rhs_quantization_max_val, "output_quantization_axis",
        output_quantization_axis, "output_quantization_min_val",
        output_quantization_min_val, "output_quantization_max_val",
        output_quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantized_dot_eager_fallback(
          lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points,
          output_scales, output_zero_points, Tout=Tout,
          lhs_quantization_axis=lhs_quantization_axis,
          lhs_quantization_min_val=lhs_quantization_min_val,
          lhs_quantization_max_val=lhs_quantization_max_val,
          rhs_quantization_axis=rhs_quantization_axis,
          rhs_quantization_min_val=rhs_quantization_min_val,
          rhs_quantization_max_val=rhs_quantization_max_val,
          output_quantization_axis=output_quantization_axis,
          output_quantization_min_val=output_quantization_min_val,
          output_quantization_max_val=output_quantization_max_val, name=name,
          ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  lhs_quantization_min_val = _execute.make_int(lhs_quantization_min_val, "lhs_quantization_min_val")
  lhs_quantization_max_val = _execute.make_int(lhs_quantization_max_val, "lhs_quantization_max_val")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if lhs_quantization_axis is None:
    lhs_quantization_axis = -1
  lhs_quantization_axis = _execute.make_int(lhs_quantization_axis, "lhs_quantization_axis")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantizedDot", lhs=lhs, rhs=rhs, lhs_scales=lhs_scales,
                               lhs_zero_points=lhs_zero_points,
                               rhs_scales=rhs_scales,
                               rhs_zero_points=rhs_zero_points,
                               output_scales=output_scales,
                               output_zero_points=output_zero_points,
                               Tout=Tout,
                               lhs_quantization_min_val=lhs_quantization_min_val,
                               lhs_quantization_max_val=lhs_quantization_max_val,
                               rhs_quantization_min_val=rhs_quantization_min_val,
                               rhs_quantization_max_val=rhs_quantization_max_val,
                               output_quantization_min_val=output_quantization_min_val,
                               output_quantization_max_val=output_quantization_max_val,
                               lhs_quantization_axis=lhs_quantization_axis,
                               rhs_quantization_axis=rhs_quantization_axis,
                               output_quantization_axis=output_quantization_axis,
                               name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tin", _op._get_attr_type("Tin"), "Tout",
              _op._get_attr_type("Tout"), "lhs_quantization_axis",
              _op._get_attr_int("lhs_quantization_axis"),
              "lhs_quantization_min_val",
              _op._get_attr_int("lhs_quantization_min_val"),
              "lhs_quantization_max_val",
              _op._get_attr_int("lhs_quantization_max_val"),
              "rhs_quantization_axis",
              _op._get_attr_int("rhs_quantization_axis"),
              "rhs_quantization_min_val",
              _op._get_attr_int("rhs_quantization_min_val"),
              "rhs_quantization_max_val",
              _op._get_attr_int("rhs_quantization_max_val"),
              "output_quantization_axis",
              _op._get_attr_int("output_quantization_axis"),
              "output_quantization_min_val",
              _op._get_attr_int("output_quantization_min_val"),
              "output_quantization_max_val",
              _op._get_attr_int("output_quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantizedDot", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantizedDot = tf_export("raw_ops.UniformQuantizedDot")(_ops.to_raw_op(uniform_quantized_dot))


def uniform_quantized_dot_eager_fallback(lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points, Tout, lhs_quantization_min_val, lhs_quantization_max_val, rhs_quantization_min_val, rhs_quantization_max_val, output_quantization_min_val, output_quantization_max_val, lhs_quantization_axis, rhs_quantization_axis, output_quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  lhs_quantization_min_val = _execute.make_int(lhs_quantization_min_val, "lhs_quantization_min_val")
  lhs_quantization_max_val = _execute.make_int(lhs_quantization_max_val, "lhs_quantization_max_val")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if lhs_quantization_axis is None:
    lhs_quantization_axis = -1
  lhs_quantization_axis = _execute.make_int(lhs_quantization_axis, "lhs_quantization_axis")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _attr_Tin, _inputs_Tin = _execute.args_to_matching_eager([lhs, rhs], ctx, [_dtypes.qint8, ])
  (lhs, rhs) = _inputs_Tin
  lhs_scales = _ops.convert_to_tensor(lhs_scales, _dtypes.float32)
  lhs_zero_points = _ops.convert_to_tensor(lhs_zero_points, _dtypes.int32)
  rhs_scales = _ops.convert_to_tensor(rhs_scales, _dtypes.float32)
  rhs_zero_points = _ops.convert_to_tensor(rhs_zero_points, _dtypes.int32)
  output_scales = _ops.convert_to_tensor(output_scales, _dtypes.float32)
  output_zero_points = _ops.convert_to_tensor(output_zero_points, _dtypes.int32)
  _inputs_flat = [lhs, rhs, lhs_scales, lhs_zero_points, rhs_scales, rhs_zero_points, output_scales, output_zero_points]
  _attrs = ("Tin", _attr_Tin, "Tout", Tout, "lhs_quantization_axis",
  lhs_quantization_axis, "lhs_quantization_min_val", lhs_quantization_min_val,
  "lhs_quantization_max_val", lhs_quantization_max_val,
  "rhs_quantization_axis", rhs_quantization_axis, "rhs_quantization_min_val",
  rhs_quantization_min_val, "rhs_quantization_max_val",
  rhs_quantization_max_val, "output_quantization_axis",
  output_quantization_axis, "output_quantization_min_val",
  output_quantization_min_val, "output_quantization_max_val",
  output_quantization_max_val)
  _result = _execute.execute(b"UniformQuantizedDot", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=ctx, name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantizedDot", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_quantized_dot_hybrid(lhs, rhs, rhs_scales, rhs_zero_points, Tout, rhs_quantization_min_val, rhs_quantization_max_val, rhs_quantization_axis=-1, name=None):
  r"""Perform hybrid quantized dot of float Tensor `lhs` and quantized Tensor `rhs`.

  Given float `lhs` and quantized `rhs`, internally performs quantization on `lhs`, and then performs quantized dot on quantized lhs and `rhs`.
  The internal quantization on `lhs` is a quantization to qint8, dynamic range, per-batch (per-axis along axis 0), asymmetric, and not narrow range (the range is [-128, 127]).
  `lhs` and `rhs` must be 2D Tensors and the lhs.dim_size(1) must match rhs.dim_size(0).
  `rhs` must be quantized Tensor, where its data value is quantized using the formula:
  quantized_data = clip(original_data / scale + zero_point, quantization_min_val, quantization_max_val).

  Args:
    lhs: A `Tensor`. Must be one of the following types: `float32`.
      Must be a 2D Tensor of Tlhs.
    rhs: A `Tensor`. Must be one of the following types: `qint8`.
      Must be a 2D Tensor of Trhs.
    rhs_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale when quantizing original data that rhs represents.
      Must be a scalar Tensor (per-tensor quantization) or 1D Tensor of size (rhs.dim_size(1),) (per-channel quantization).
    rhs_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point when quantizing original data that rhs represents.
      Same shape condition as rhs_scales.
    Tout: A `tf.DType` from: `tf.float32`. The type of output Tensor.
    rhs_quantization_min_val: An `int`.
      The min value of the quantized data stored in rhs.
      For example, if Trhs is qint8, this must be set to -127 if narrow range quantized or -128 if not.
    rhs_quantization_max_val: An `int`.
      The max value of the quantized data stored in rhs.
      For example, if Trhs is qint8, this must be set to 127.
    rhs_quantization_axis: An optional `int`. Defaults to `-1`.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization.
      For dot op rhs, only per-tensor quantization or per-channel quantization along dimension 1 is supported.
      Thus, this attribute must be set to -1 or 1. Other values are rejected.
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformQuantizedDotHybrid", name, lhs, rhs, rhs_scales,
        rhs_zero_points, "Tout", Tout, "rhs_quantization_axis",
        rhs_quantization_axis, "rhs_quantization_min_val",
        rhs_quantization_min_val, "rhs_quantization_max_val",
        rhs_quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_quantized_dot_hybrid_eager_fallback(
          lhs, rhs, rhs_scales, rhs_zero_points, Tout=Tout,
          rhs_quantization_axis=rhs_quantization_axis,
          rhs_quantization_min_val=rhs_quantization_min_val,
          rhs_quantization_max_val=rhs_quantization_max_val, name=name,
          ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformQuantizedDotHybrid", lhs=lhs, rhs=rhs, rhs_scales=rhs_scales,
                                     rhs_zero_points=rhs_zero_points,
                                     Tout=Tout,
                                     rhs_quantization_min_val=rhs_quantization_min_val,
                                     rhs_quantization_max_val=rhs_quantization_max_val,
                                     rhs_quantization_axis=rhs_quantization_axis,
                                     name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tlhs", _op._get_attr_type("Tlhs"), "Trhs",
              _op._get_attr_type("Trhs"), "Tout", _op._get_attr_type("Tout"),
              "rhs_quantization_axis",
              _op._get_attr_int("rhs_quantization_axis"),
              "rhs_quantization_min_val",
              _op._get_attr_int("rhs_quantization_min_val"),
              "rhs_quantization_max_val",
              _op._get_attr_int("rhs_quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformQuantizedDotHybrid", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformQuantizedDotHybrid = tf_export("raw_ops.UniformQuantizedDotHybrid")(_ops.to_raw_op(uniform_quantized_dot_hybrid))


def uniform_quantized_dot_hybrid_eager_fallback(lhs, rhs, rhs_scales, rhs_zero_points, Tout, rhs_quantization_min_val, rhs_quantization_max_val, rhs_quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  rhs_quantization_min_val = _execute.make_int(rhs_quantization_min_val, "rhs_quantization_min_val")
  rhs_quantization_max_val = _execute.make_int(rhs_quantization_max_val, "rhs_quantization_max_val")
  if rhs_quantization_axis is None:
    rhs_quantization_axis = -1
  rhs_quantization_axis = _execute.make_int(rhs_quantization_axis, "rhs_quantization_axis")
  _attr_Tlhs, (lhs,) = _execute.args_to_matching_eager([lhs], ctx, [_dtypes.float32, ])
  _attr_Trhs, (rhs,) = _execute.args_to_matching_eager([rhs], ctx, [_dtypes.qint8, ])
  rhs_scales = _ops.convert_to_tensor(rhs_scales, _dtypes.float32)
  rhs_zero_points = _ops.convert_to_tensor(rhs_zero_points, _dtypes.int32)
  _inputs_flat = [lhs, rhs, rhs_scales, rhs_zero_points]
  _attrs = ("Tlhs", _attr_Tlhs, "Trhs", _attr_Trhs, "Tout", Tout,
  "rhs_quantization_axis", rhs_quantization_axis, "rhs_quantization_min_val",
  rhs_quantization_min_val, "rhs_quantization_max_val",
  rhs_quantization_max_val)
  _result = _execute.execute(b"UniformQuantizedDotHybrid", 1,
                             inputs=_inputs_flat, attrs=_attrs, ctx=ctx,
                             name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformQuantizedDotHybrid", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result


def uniform_requantize(input, input_scales, input_zero_points, output_scales, output_zero_points, Tout, input_quantization_min_val, input_quantization_max_val, output_quantization_min_val, output_quantization_max_val, input_quantization_axis=-1, output_quantization_axis=-1, name=None):
  r"""Given quantized tensor `input`, requantize it with new quantization parameters.

  Given quantized tensor `input`, which was quantized using {input_scales, input_zero_points, input_quantization_axis, input_quantization_min_val, input_quantization_max_val},
  requantize it to a tensor, which is quantized using {output_scales, output_zero_points, output_quantization_axis, output_quantization_min_val, output_quantization_max_val}.
  The requantization is done by using the formula:
  output_quantized_data = clip(
    (input_quantized_data - input_zero_point) * (input_scale / output_scale) + output_zero_point,
    output_quantization_min_val,
    output_quantization_max_val)

  Per-tensor and per-axis quantization supported cases are followings:
  * per-tensor -> per-tensor
  * per-tensor -> per-axis
  * per-axis -> per-axis where input_quantization_axis equals output_quantization_axis.
  i.e. At least one among input_quantization_axis and output_quantization_axis must be -1, or two must be equal.

  Args:
    input: A `Tensor`. Must be one of the following types: `qint8`, `qint32`.
      Must be a Tensor of Tin.
    input_scales: A `Tensor` of type `float32`.
      The float value(s) used as scale(s) when quantizing original data that `input` represents.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (input.dim_size(quantization_axis),) (per-axis quantization).
    input_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) used as zero_point(s) when quantizing original data that `input` represents.
      Same shape condition as scales.
    output_scales: A `Tensor` of type `float32`.
      The float value(s) to use as new scale(s) to quantize original data that `input` represents.
      Must be a scalar Tensor if quantization_axis is -1 (per-tensor quantization), otherwise 1D Tensor of size (input.dim_size(quantization_axis),) (per-axis quantization).
    output_zero_points: A `Tensor` of type `int32`.
      The int32 value(s) to use as new zero_point(s) to quantize original data that `input` represents.
      Same shape condition as scales.
    Tout: A `tf.DType` from: `tf.qint8, tf.qint32`.
      The type of output Tensor. A tf.DType from: tf.qint8, tf.qint32
    input_quantization_min_val: An `int`.
      The quantization min value that was used when quantizing original data that `input` represents.
      The purpose of this attribute is typically (but not limited to) to indicate narrow range, where this is set to:
      `(Tin lowest) + 1` if narrow range, and `(Tin lowest)` otherwise.
      For example, if Tin is qint8, this is set to -127 if narrow range quantized or -128 if not.
    input_quantization_max_val: An `int`.
      The quantization max value that was used when quantizing original data that `input` represents.
      The purpose of this attribute is typically (but not limited to) indicate narrow range, where this is set to:
      `(Tout max)` for both narrow range and not narrow range.
      For example, if Tin is qint8, this is set to 127.
    output_quantization_min_val: An `int`.
      The new quantization min value to quantize original data that `input` represents.
    output_quantization_max_val: An `int`.
      The new quantization max value to quantize original data that `input` represents.
    input_quantization_axis: An optional `int`. Defaults to `-1`.
      The quantization axis that was used when quantizing original data that `input` represents.
      Indicates the dimension index of the tensor where per-axis quantization is applied for the slices along that dimension.
      If set to -1 (default), this indicates per-tensor quantization. Otherwise, it must be set within range [0, input.dims()).
    output_quantization_axis: An optional `int`. Defaults to `-1`.
      The new quantization axis to use to quantize original data that `input` represents.
    name: A name for the operation (optional).

  Returns:
    A `Tensor` of type `Tout`.
  """
  _ctx = _context._context or _context.context()
  tld = _ctx._thread_local_data
  if tld.is_eager:
    try:
      _result = pywrap_tfe.TFE_Py_FastPathExecute(
        _ctx, "UniformRequantize", name, input, input_scales,
        input_zero_points, output_scales, output_zero_points, "Tout", Tout,
        "input_quantization_axis", input_quantization_axis,
        "input_quantization_min_val", input_quantization_min_val,
        "input_quantization_max_val", input_quantization_max_val,
        "output_quantization_axis", output_quantization_axis,
        "output_quantization_min_val", output_quantization_min_val,
        "output_quantization_max_val", output_quantization_max_val)
      return _result
    except _core._NotOkStatusException as e:
      _ops.raise_from_not_ok_status(e, name)
    except _core._FallbackException:
      pass
    try:
      return uniform_requantize_eager_fallback(
          input, input_scales, input_zero_points, output_scales,
          output_zero_points, Tout=Tout,
          input_quantization_axis=input_quantization_axis,
          input_quantization_min_val=input_quantization_min_val,
          input_quantization_max_val=input_quantization_max_val,
          output_quantization_axis=output_quantization_axis,
          output_quantization_min_val=output_quantization_min_val,
          output_quantization_max_val=output_quantization_max_val, name=name,
          ctx=_ctx)
    except _core._SymbolicException:
      pass  # Add nodes to the TensorFlow graph.
  # Add nodes to the TensorFlow graph.
  Tout = _execute.make_type(Tout, "Tout")
  input_quantization_min_val = _execute.make_int(input_quantization_min_val, "input_quantization_min_val")
  input_quantization_max_val = _execute.make_int(input_quantization_max_val, "input_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if input_quantization_axis is None:
    input_quantization_axis = -1
  input_quantization_axis = _execute.make_int(input_quantization_axis, "input_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _, _, _op, _outputs = _op_def_library._apply_op_helper(
        "UniformRequantize", input=input, input_scales=input_scales,
                             input_zero_points=input_zero_points,
                             output_scales=output_scales,
                             output_zero_points=output_zero_points, Tout=Tout,
                             input_quantization_min_val=input_quantization_min_val,
                             input_quantization_max_val=input_quantization_max_val,
                             output_quantization_min_val=output_quantization_min_val,
                             output_quantization_max_val=output_quantization_max_val,
                             input_quantization_axis=input_quantization_axis,
                             output_quantization_axis=output_quantization_axis,
                             name=name)
  _result = _outputs[:]
  if _execute.must_record_gradient():
    _attrs = ("Tin", _op._get_attr_type("Tin"), "Tout",
              _op._get_attr_type("Tout"), "input_quantization_axis",
              _op._get_attr_int("input_quantization_axis"),
              "input_quantization_min_val",
              _op._get_attr_int("input_quantization_min_val"),
              "input_quantization_max_val",
              _op._get_attr_int("input_quantization_max_val"),
              "output_quantization_axis",
              _op._get_attr_int("output_quantization_axis"),
              "output_quantization_min_val",
              _op._get_attr_int("output_quantization_min_val"),
              "output_quantization_max_val",
              _op._get_attr_int("output_quantization_max_val"))
    _inputs_flat = _op.inputs
    _execute.record_gradient(
        "UniformRequantize", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result

UniformRequantize = tf_export("raw_ops.UniformRequantize")(_ops.to_raw_op(uniform_requantize))


def uniform_requantize_eager_fallback(input, input_scales, input_zero_points, output_scales, output_zero_points, Tout, input_quantization_min_val, input_quantization_max_val, output_quantization_min_val, output_quantization_max_val, input_quantization_axis, output_quantization_axis, name, ctx):
  Tout = _execute.make_type(Tout, "Tout")
  input_quantization_min_val = _execute.make_int(input_quantization_min_val, "input_quantization_min_val")
  input_quantization_max_val = _execute.make_int(input_quantization_max_val, "input_quantization_max_val")
  output_quantization_min_val = _execute.make_int(output_quantization_min_val, "output_quantization_min_val")
  output_quantization_max_val = _execute.make_int(output_quantization_max_val, "output_quantization_max_val")
  if input_quantization_axis is None:
    input_quantization_axis = -1
  input_quantization_axis = _execute.make_int(input_quantization_axis, "input_quantization_axis")
  if output_quantization_axis is None:
    output_quantization_axis = -1
  output_quantization_axis = _execute.make_int(output_quantization_axis, "output_quantization_axis")
  _attr_Tin, (input,) = _execute.args_to_matching_eager([input], ctx, [_dtypes.qint8, _dtypes.qint32, ])
  input_scales = _ops.convert_to_tensor(input_scales, _dtypes.float32)
  input_zero_points = _ops.convert_to_tensor(input_zero_points, _dtypes.int32)
  output_scales = _ops.convert_to_tensor(output_scales, _dtypes.float32)
  output_zero_points = _ops.convert_to_tensor(output_zero_points, _dtypes.int32)
  _inputs_flat = [input, input_scales, input_zero_points, output_scales, output_zero_points]
  _attrs = ("Tin", _attr_Tin, "Tout", Tout, "input_quantization_axis",
  input_quantization_axis, "input_quantization_min_val",
  input_quantization_min_val, "input_quantization_max_val",
  input_quantization_max_val, "output_quantization_axis",
  output_quantization_axis, "output_quantization_min_val",
  output_quantization_min_val, "output_quantization_max_val",
  output_quantization_max_val)
  _result = _execute.execute(b"UniformRequantize", 1, inputs=_inputs_flat,
                             attrs=_attrs, ctx=ctx, name=name)
  if _execute.must_record_gradient():
    _execute.record_gradient(
        "UniformRequantize", _inputs_flat, _attrs, _result)
  _result, = _result
  return _result




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/lite/toco/model_flags.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.lite.toco import types_pb2 as tensorflow_dot_lite_dot_toco_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tensorflow/lite/toco/model_flags.proto\x12\x04toco\x1a tensorflow/lite/toco/types.proto\"5\n\x0fInputArrayShape\x12\x0c\n\x04\x64ims\x18\x02 \x03(\x05\x12\x14\n\x0cunknown_rank\x18\x03 \x01(\x08\"\x8f\x01\n\nInputArray\x12\x0c\n\x04name\x18\x01 \x01(\t\x12$\n\x05shape\x18\x06 \x01(\x0b\x32\x15.toco.InputArrayShape\x12\x12\n\nmean_value\x18\x03 \x01(\x02\x12\x14\n\tstd_value\x18\x04 \x01(\x02:\x01\x31\x12#\n\tdata_type\x18\x05 \x01(\x0e\x32\x10.toco.IODataType\"t\n\x08RnnState\x12\x13\n\x0bstate_array\x18\x01 \x01(\t\x12\x1e\n\x16\x62\x61\x63k_edge_source_array\x18\x02 \x01(\t\x12\x13\n\x0b\x64iscardable\x18\x05 \x01(\x08\x12\x0c\n\x04size\x18\x03 \x01(\x05\x12\x10\n\x08num_dims\x18\x04 \x01(\x05\"\xef\x01\n\x0f\x41rraysExtraInfo\x12,\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x1b.toco.ArraysExtraInfo.Entry\x1a\xad\x01\n\x05\x45ntry\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0bname_regexp\x18\x07 \x01(\t\x12\x0b\n\x03min\x18\x02 \x01(\x01\x12\x0b\n\x03max\x18\x03 \x01(\x01\x12#\n\tdata_type\x18\x04 \x01(\x0e\x32\x10.toco.IODataType\x12$\n\x05shape\x18\x05 \x01(\x0b\x32\x15.toco.InputArrayShape\x12\x1c\n\x14\x63onstant_float_value\x18\x06 \x01(\x02\"\xc6\x05\n\nModelFlags\x12&\n\x0cinput_arrays\x18\x01 \x03(\x0b\x32\x10.toco.InputArray\x12\x15\n\routput_arrays\x18\x02 \x03(\t\x12\x1d\n\x15\x63ontrol_output_arrays\x18\x18 \x03(\t\x12\x16\n\x0evariable_batch\x18\n \x01(\x08\x12\"\n\nrnn_states\x18\x0c \x03(\x0b\x32\x0e.toco.RnnState\x12\x31\n\x0cmodel_checks\x18\x0e \x03(\x0b\x32\x1b.toco.ModelFlags.ModelCheck\x12 \n\x18\x61llow_nonexistent_arrays\x18\x10 \x01(\x08\x12\x1d\n\x15\x61llow_nonascii_arrays\x18\x11 \x01(\x08\x12\x30\n\x11\x61rrays_extra_info\x18\x12 \x01(\x0b\x32\x15.toco.ArraysExtraInfo\x12(\n\x1a\x63hange_concat_input_ranges\x18\x13 \x01(\x08:\x04true\x12\x17\n\x0fsaved_model_dir\x18\x14 \x01(\t\x12\x1b\n\x13saved_model_version\x18\x15 \x01(\x05\x12\x18\n\x10saved_model_tags\x18\x16 \x03(\t\x12\"\n\x1asaved_model_exported_names\x18\x17 \x03(\t\x12\x16\n\x0euse_hlo_import\x18\x19 \x01(\x08\x12\x33\n\rhlo_file_type\x18\x1a \x01(\x0e\x32\x1c.toco.ModelFlags.HloFileType\x1aT\n\nModelCheck\x12\x18\n\ncount_type\x18\x01 \x01(\t:\x04None\x12\x15\n\tcount_min\x18\x02 \x01(\x05:\x02-1\x12\x15\n\tcount_max\x18\x03 \x01(\x05:\x02-1\"7\n\x0bHloFileType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08HLO_TEXT\x10\x01\x12\r\n\tHLO_PROTO\x10\x02')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.lite.toco.model_flags_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INPUTARRAYSHAPE._serialized_start=82
  _INPUTARRAYSHAPE._serialized_end=135
  _INPUTARRAY._serialized_start=138
  _INPUTARRAY._serialized_end=281
  _RNNSTATE._serialized_start=283
  _RNNSTATE._serialized_end=399
  _ARRAYSEXTRAINFO._serialized_start=402
  _ARRAYSEXTRAINFO._serialized_end=641
  _ARRAYSEXTRAINFO_ENTRY._serialized_start=468
  _ARRAYSEXTRAINFO_ENTRY._serialized_end=641
  _MODELFLAGS._serialized_start=644
  _MODELFLAGS._serialized_end=1354
  _MODELFLAGS_MODELCHECK._serialized_start=1213
  _MODELFLAGS_MODELCHECK._serialized_end=1297
  _MODELFLAGS_HLOFILETYPE._serialized_start=1299
  _MODELFLAGS_HLOFILETYPE._serialized_end=1354
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/framework/optimized_function_graph.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import graph_pb2 as tensorflow_dot_core_dot_framework_dot_graph__pb2
from tensorflow.core.framework import types_pb2 as tensorflow_dot_core_dot_framework_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8tensorflow/core/framework/optimized_function_graph.proto\x12\ntensorflow\x1a%tensorflow/core/framework/graph.proto\x1a%tensorflow/core/framework/types.proto\"\xb4\x02\n\x16OptimizedFunctionGraph\x12\x0c\n\x04name\x18\x01 \x01(\t\x12,\n\x0e\x66unction_graph\x18\x02 \x01(\x0b\x32\x14.tensorflow.GraphDef\x12^\n\x18node_name_to_control_ret\x18\x03 \x03(\x0b\x32<.tensorflow.OptimizedFunctionGraph.NodeNameToControlRetEntry\x12\'\n\tret_types\x18\x04 \x03(\x0e\x32\x14.tensorflow.DataType\x12\x18\n\x10num_return_nodes\x18\x05 \x01(\r\x1a;\n\x19NodeNameToControlRetEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.framework.optimized_function_graph_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._options = None
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._serialized_options = b'8\001'
  _OPTIMIZEDFUNCTIONGRAPH._serialized_start=151
  _OPTIMIZEDFUNCTIONGRAPH._serialized_end=459
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._serialized_start=400
  _OPTIMIZEDFUNCTIONGRAPH_NODENAMETOCONTROLRETENTRY._serialized_end=459
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/function/trace_type/serialization_test.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.function.trace_type import serialization_pb2 as tensorflow_dot_core_dot_function_dot_trace__type_dot_serialization__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<tensorflow/core/function/trace_type/serialization_test.proto\x12\x36tensorflow.core.function.trace_type.serialization_test\x1a\x37tensorflow/core/function/trace_type/serialization.proto\"5\n\x16MyCustomRepresentation\x12\r\n\x05index\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\"u\n\x19MyCompositeRepresentation\x12X\n\x08\x65lements\x18\x01 \x03(\x0b\x32\x46.tensorflow.core.function.trace_type.serialization.SerializedTraceType\"(\n\x1aMyMultiClassRepresentation\x12\n\n\x02id\x18\x01 \x01(\x05')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.function.trace_type.serialization_test_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MYCUSTOMREPRESENTATION._serialized_start=177
  _MYCUSTOMREPRESENTATION._serialized_end=230
  _MYCOMPOSITEREPRESENTATION._serialized_start=232
  _MYCOMPOSITEREPRESENTATION._serialized_end=349
  _MYMULTICLASSREPRESENTATION._serialized_start=351
  _MYMULTICLASSREPRESENTATION._serialized_end=391
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/debug_event.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import tensor_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__pb2
from tensorflow.core.protobuf import graph_debug_info_pb2 as tensorflow_dot_core_dot_protobuf_dot_graph__debug__info__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tensorflow/core/protobuf/debug_event.proto\x12\ntensorflow\x1a&tensorflow/core/framework/tensor.proto\x1a/tensorflow/core/protobuf/graph_debug_info.proto\"\xfe\x03\n\nDebugEvent\x12\x11\n\twall_time\x18\x01 \x01(\x01\x12\x0c\n\x04step\x18\x02 \x01(\x03\x12\x33\n\x0e\x64\x65\x62ug_metadata\x18\x03 \x01(\x0b\x32\x19.tensorflow.DebugMetadataH\x00\x12-\n\x0bsource_file\x18\x04 \x01(\x0b\x32\x16.tensorflow.SourceFileH\x00\x12;\n\x13stack_frame_with_id\x18\x06 \x01(\x0b\x32\x1c.tensorflow.StackFrameWithIdH\x00\x12\x38\n\x11graph_op_creation\x18\x07 \x01(\x0b\x32\x1b.tensorflow.GraphOpCreationH\x00\x12\x33\n\x0e\x64\x65\x62ugged_graph\x18\x08 \x01(\x0b\x32\x19.tensorflow.DebuggedGraphH\x00\x12*\n\texecution\x18\t \x01(\x0b\x32\x15.tensorflow.ExecutionH\x00\x12@\n\x15graph_execution_trace\x18\n \x01(\x0b\x32\x1f.tensorflow.GraphExecutionTraceH\x00\x12\x12\n\x08graph_id\x18\x0b \x01(\tH\x00\x12\x35\n\x0f\x64\x65\x62ugged_device\x18\x0c \x01(\x0b\x32\x1a.tensorflow.DebuggedDeviceH\x00\x42\x06\n\x04what\"W\n\rDebugMetadata\x12\x1a\n\x12tensorflow_version\x18\x01 \x01(\t\x12\x14\n\x0c\x66ile_version\x18\x02 \x01(\t\x12\x14\n\x0ctfdbg_run_id\x18\x03 \x01(\t\"A\n\nSourceFile\x12\x11\n\tfile_path\x18\x01 \x01(\t\x12\x11\n\thost_name\x18\x02 \x01(\t\x12\r\n\x05lines\x18\x03 \x03(\t\"]\n\x10StackFrameWithId\x12\n\n\x02id\x18\x01 \x01(\t\x12=\n\rfile_line_col\x18\x02 \x01(\x0b\x32&.tensorflow.GraphDebugInfo.FileLineCol\":\n\x0c\x43odeLocation\x12\x11\n\thost_name\x18\x01 \x01(\t\x12\x17\n\x0fstack_frame_ids\x18\x02 \x03(\t\"\xe4\x01\n\x0fGraphOpCreation\x12\x0f\n\x07op_type\x18\x01 \x01(\t\x12\x0f\n\x07op_name\x18\x02 \x01(\t\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12\x10\n\x08graph_id\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65vice_name\x18\x05 \x01(\t\x12\x13\n\x0binput_names\x18\x06 \x03(\t\x12\x13\n\x0bnum_outputs\x18\x07 \x01(\x05\x12/\n\rcode_location\x18\x08 \x01(\x0b\x32\x18.tensorflow.CodeLocation\x12\x19\n\x11output_tensor_ids\x18\t \x03(\x05\"\xa5\x01\n\rDebuggedGraph\x12\x10\n\x08graph_id\x18\x01 \x01(\t\x12\x12\n\ngraph_name\x18\x02 \x01(\t\x12\x18\n\x10instrumented_ops\x18\x03 \x03(\t\x12\x1a\n\x12original_graph_def\x18\x04 \x01(\x0c\x12\x1e\n\x16instrumented_graph_def\x18\x05 \x01(\x0c\x12\x18\n\x10outer_context_id\x18\x06 \x01(\t\"8\n\x0e\x44\x65\x62uggedDevice\x12\x13\n\x0b\x64\x65vice_name\x18\x01 \x01(\t\x12\x11\n\tdevice_id\x18\x02 \x01(\x05\"\xb3\x02\n\tExecution\x12\x0f\n\x07op_type\x18\x01 \x01(\t\x12\x13\n\x0bnum_outputs\x18\x02 \x01(\x05\x12\x10\n\x08graph_id\x18\x03 \x01(\t\x12\x18\n\x10input_tensor_ids\x18\x04 \x03(\x03\x12\x19\n\x11output_tensor_ids\x18\x05 \x03(\x03\x12\x36\n\x11tensor_debug_mode\x18\x06 \x01(\x0e\x32\x1b.tensorflow.TensorDebugMode\x12.\n\rtensor_protos\x18\x07 \x03(\x0b\x32\x17.tensorflow.TensorProto\x12/\n\rcode_location\x18\x08 \x01(\x0b\x32\x18.tensorflow.CodeLocation\x12 \n\x18output_tensor_device_ids\x18\t \x03(\x05\"\xd1\x01\n\x13GraphExecutionTrace\x12\x18\n\x10tfdbg_context_id\x18\x01 \x01(\t\x12\x0f\n\x07op_name\x18\x02 \x01(\t\x12\x13\n\x0boutput_slot\x18\x03 \x01(\x05\x12\x36\n\x11tensor_debug_mode\x18\x04 \x01(\x0e\x32\x1b.tensorflow.TensorDebugMode\x12-\n\x0ctensor_proto\x18\x05 \x01(\x0b\x32\x17.tensorflow.TensorProto\x12\x13\n\x0b\x64\x65vice_name\x18\x06 \x01(\t*\xb6\x01\n\x0fTensorDebugMode\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\r\n\tNO_TENSOR\x10\x01\x12\x0f\n\x0b\x43URT_HEALTH\x10\x02\x12\x12\n\x0e\x43ONCISE_HEALTH\x10\x03\x12\x0f\n\x0b\x46ULL_HEALTH\x10\x04\x12\t\n\x05SHAPE\x10\x05\x12\x11\n\rFULL_NUMERICS\x10\x06\x12\x0f\n\x0b\x46ULL_TENSOR\x10\x07\x12\x1e\n\x1aREDUCE_INF_NAN_THREE_SLOTS\x10\x08\x42\x83\x01\n\x13org.tensorflow.utilB\x10\x44\x65\x62ugEventProtosP\x01ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.debug_event_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023org.tensorflow.utilB\020DebugEventProtosP\001ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _TENSORDEBUGMODE._serialized_start=1951
  _TENSORDEBUGMODE._serialized_end=2133
  _DEBUGEVENT._serialized_start=148
  _DEBUGEVENT._serialized_end=658
  _DEBUGMETADATA._serialized_start=660
  _DEBUGMETADATA._serialized_end=747
  _SOURCEFILE._serialized_start=749
  _SOURCEFILE._serialized_end=814
  _STACKFRAMEWITHID._serialized_start=816
  _STACKFRAMEWITHID._serialized_end=909
  _CODELOCATION._serialized_start=911
  _CODELOCATION._serialized_end=969
  _GRAPHOPCREATION._serialized_start=972
  _GRAPHOPCREATION._serialized_end=1200
  _DEBUGGEDGRAPH._serialized_start=1203
  _DEBUGGEDGRAPH._serialized_end=1368
  _DEBUGGEDDEVICE._serialized_start=1370
  _DEBUGGEDDEVICE._serialized_end=1426
  _EXECUTION._serialized_start=1429
  _EXECUTION._serialized_end=1736
  _GRAPHEXECUTIONTRACE._serialized_start=1739
  _GRAPHEXECUTIONTRACE._serialized_end=1948
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/framework/attr_value.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tensorflow.core.framework import tensor_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__pb2
from tensorflow.core.framework import tensor_shape_pb2 as tensorflow_dot_core_dot_framework_dot_tensor__shape__pb2
from tensorflow.core.framework import types_pb2 as tensorflow_dot_core_dot_framework_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tensorflow/core/framework/attr_value.proto\x12\ntensorflow\x1a&tensorflow/core/framework/tensor.proto\x1a,tensorflow/core/framework/tensor_shape.proto\x1a%tensorflow/core/framework/types.proto\"\xa6\x04\n\tAttrValue\x12\x0b\n\x01s\x18\x02 \x01(\x0cH\x00\x12\x0b\n\x01i\x18\x03 \x01(\x03H\x00\x12\x0b\n\x01\x66\x18\x04 \x01(\x02H\x00\x12\x0b\n\x01\x62\x18\x05 \x01(\x08H\x00\x12$\n\x04type\x18\x06 \x01(\x0e\x32\x14.tensorflow.DataTypeH\x00\x12-\n\x05shape\x18\x07 \x01(\x0b\x32\x1c.tensorflow.TensorShapeProtoH\x00\x12)\n\x06tensor\x18\x08 \x01(\x0b\x32\x17.tensorflow.TensorProtoH\x00\x12/\n\x04list\x18\x01 \x01(\x0b\x32\x1f.tensorflow.AttrValue.ListValueH\x00\x12(\n\x04\x66unc\x18\n \x01(\x0b\x32\x18.tensorflow.NameAttrListH\x00\x12\x15\n\x0bplaceholder\x18\t \x01(\tH\x00\x1a\xe9\x01\n\tListValue\x12\t\n\x01s\x18\x02 \x03(\x0c\x12\r\n\x01i\x18\x03 \x03(\x03\x42\x02\x10\x01\x12\r\n\x01\x66\x18\x04 \x03(\x02\x42\x02\x10\x01\x12\r\n\x01\x62\x18\x05 \x03(\x08\x42\x02\x10\x01\x12&\n\x04type\x18\x06 \x03(\x0e\x32\x14.tensorflow.DataTypeB\x02\x10\x01\x12+\n\x05shape\x18\x07 \x03(\x0b\x32\x1c.tensorflow.TensorShapeProto\x12\'\n\x06tensor\x18\x08 \x03(\x0b\x32\x17.tensorflow.TensorProto\x12&\n\x04\x66unc\x18\t \x03(\x0b\x32\x18.tensorflow.NameAttrListB\x07\n\x05value\"\x92\x01\n\x0cNameAttrList\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\x04\x61ttr\x18\x02 \x03(\x0b\x32\".tensorflow.NameAttrList.AttrEntry\x1a\x42\n\tAttrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.tensorflow.AttrValue:\x02\x38\x01\x42\x83\x01\n\x18org.tensorflow.frameworkB\x0f\x41ttrValueProtosP\x01ZQgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/attr_value_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.framework.attr_value_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030org.tensorflow.frameworkB\017AttrValueProtosP\001ZQgithub.com/tensorflow/tensorflow/tensorflow/go/core/framework/attr_value_go_proto\370\001\001'
  _ATTRVALUE_LISTVALUE.fields_by_name['i']._options = None
  _ATTRVALUE_LISTVALUE.fields_by_name['i']._serialized_options = b'\020\001'
  _ATTRVALUE_LISTVALUE.fields_by_name['f']._options = None
  _ATTRVALUE_LISTVALUE.fields_by_name['f']._serialized_options = b'\020\001'
  _ATTRVALUE_LISTVALUE.fields_by_name['b']._options = None
  _ATTRVALUE_LISTVALUE.fields_by_name['b']._serialized_options = b'\020\001'
  _ATTRVALUE_LISTVALUE.fields_by_name['type']._options = None
  _ATTRVALUE_LISTVALUE.fields_by_name['type']._serialized_options = b'\020\001'
  _NAMEATTRLIST_ATTRENTRY._options = None
  _NAMEATTRLIST_ATTRENTRY._serialized_options = b'8\001'
  _ATTRVALUE._serialized_start=184
  _ATTRVALUE._serialized_end=734
  _ATTRVALUE_LISTVALUE._serialized_start=492
  _ATTRVALUE_LISTVALUE._serialized_end=725
  _NAMEATTRLIST._serialized_start=737
  _NAMEATTRLIST._serialized_end=883
  _NAMEATTRLIST_ATTRENTRY._serialized_start=817
  _NAMEATTRLIST_ATTRENTRY._serialized_end=883
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/device_properties.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0tensorflow/core/protobuf/device_properties.proto\x12\ntensorflow\"\x90\x03\n\x10\x44\x65viceProperties\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0e\n\x06vendor\x18\x02 \x01(\t\x12\r\n\x05model\x18\x03 \x01(\t\x12\x11\n\tfrequency\x18\x04 \x01(\x03\x12\x11\n\tnum_cores\x18\x05 \x01(\x03\x12\x42\n\x0b\x65nvironment\x18\x06 \x03(\x0b\x32-.tensorflow.DeviceProperties.EnvironmentEntry\x12\x15\n\rnum_registers\x18\x07 \x01(\x03\x12\x15\n\rl1_cache_size\x18\x08 \x01(\x03\x12\x15\n\rl2_cache_size\x18\t \x01(\x03\x12\x15\n\rl3_cache_size\x18\n \x01(\x03\x12-\n%shared_memory_size_per_multiprocessor\x18\x0b \x01(\x03\x12\x13\n\x0bmemory_size\x18\x0c \x01(\x03\x12\x11\n\tbandwidth\x18\r \x01(\x03\x1a\x32\n\x10\x45nvironmentEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"M\n\x0bNamedDevice\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x30\n\nproperties\x18\x02 \x01(\x0b\x32\x1c.tensorflow.DevicePropertiesBrB\x16\x44\x65vicePropertiesProtosZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tensorflow.core.protobuf.device_properties_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'B\026DevicePropertiesProtosZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto\370\001\001'
  _DEVICEPROPERTIES_ENVIRONMENTENTRY._options = None
  _DEVICEPROPERTIES_ENVIRONMENTENTRY._serialized_options = b'8\001'
  _DEVICEPROPERTIES._serialized_start=65
  _DEVICEPROPERTIES._serialized_end=465
  _DEVICEPROPERTIES_ENVIRONMENTENTRY._serialized_start=415
  _DEVICEPROPERTIES_ENVIRONMENTENTRY._serialized_end=465
  _NAMEDDEVICE._serialized_start=467
  _NAMEDDEVICE._serialized_end=544
# @@protoc_insertion_point(module_scope)
